@@ -82,17 +82,17 @@ typedef uchar bool;
 typedef void *jsobjtype;
 
 extern const char *jsSourceFile; // sourcefile providing the javascript
-extern int jsLineno; // line number
+extern int jsLineno; // js line number
+extern int undo1line; // special undo line number
+extern int imapfetch; // fetch this many envelopes in imap
+extern char frameContent[];
+	extern const char startWindowJS[];
+	extern const char deminJS[];
+	extern const char sharedJS[];
+// this is crude but it works.
+#define WithDebugging (strlen(deminJS) > 5000)
 
-/*********************************************************************
-duktape is not threadsafe, so says the programmer's guide.
-So javascript runs in the foreground as we push buttons etc,
-or, there is at most one thread running javascript in the background,
-if we choose to implement that.
-Here is the js background thread. It is 0 if no thread is running.
-*********************************************************************/
-
-extern pthread_t jsbt;
+#define cssDebugFile "/tmp/css"
 
 enum ej_proptype {
 	EJ_PROP_NONE,
@@ -135,6 +135,7 @@ typedef uchar *pst;		/* perl string */
 #define InternalCodeChar '\2'
 #define InternalCodeCharAlternate '\1'
 #define TableCellChar '\3'
+#define DataCellChar '\4'
 
 /* How long can an absolute path be? */
 #define ABSPATH 1024 // max length of an absolute pathname
@@ -144,6 +145,7 @@ typedef uchar *pst;		/* perl string */
 #define MAXPROTLEN 12
 #define MAXUSERPASS 80 // user name or password
 #define MAXACCOUNT 100 // number of email accounts
+#define MAXCC 10 // max automatic cc or bcc that you can include in a mail account
 #define MAXAGENT 50 // number of user agents
 #define MAXMIME 40 // number of mime types
 #define MAXPROXY 200 // number of proxy entries
@@ -182,6 +184,7 @@ extern char emptyString[];	/* use this whenever you would use "" */
  * Tell them you're Explorer, and walk right in.
  * Anyways, this array holds up to 10 user agent strings. */
 extern char *userAgents[], *currentAgent;
+extern int agentIndex;
 extern char *newlocation;
 extern int newloc_d; /* delay */
 extern bool newloc_r; /* location replaces this page */
@@ -241,12 +244,14 @@ long long hcl;			/* http content length */
 	int newloc_d;
 };
 
-struct MACCOUNT {		/* pop3 account */
+struct MACCOUNT { // email account, pop3 or imap
 	char *login, *password, *from, *reply;
 	char *inurl, *outurl;
 	int inport, outport;
 	uchar inssl, outssl;
 	bool nofetch, imap, secure;
+	const char *cclist[MAXCC + 1]; // extra cc directives
+	uchar cctype[MAXCC]; // cc or bcc or attach
 };
 extern struct MACCOUNT accounts[];	/* all the email accounts */
 extern int maxAccount;		/* how many email accounts specified */
@@ -280,6 +285,7 @@ extern bool debugClone, debugEvent, debugThrow, debugCSS;
 extern bool demin; // deminimize javascript
 extern bool uvw; // trace points
 extern bool gotimers; // run javascript timers
+extern int timerspeed; // slowdown factor for javascript timers
 extern int rr_interval; // rerender the screen after this many seconds
 extern FILE *debugFile;
 extern char *debugFileName;
@@ -294,11 +300,14 @@ extern bool sqlPresent;		/* Was edbrowse compiled with SQL built in? */
 extern bool curlActive; // is curl running?
 extern bool ismc;		/* Is the program running as a mail client? */
 extern bool isimap;		/* Is the program running as an imap client? */
+// next two variables work around curl bug 7284
 extern bool down_bg;		// download in background
 extern bool down_jsbg;		// download javascript in background
-extern char whichproc; // which edbrowse-xx process
+extern bool re_utf8, endMarks;
 extern char showProgress; // feedback as a file is downloaded
+extern char selsep; // selection separator
 extern char eb_language[];		/* edbrowse language, determined by $LANG */
+extern  const char *const supported_languages[];
 extern int eb_lang; // encoded version of the above, for languages that we recognize
 extern bool cons_utf8;		/* does the console expect utf8? */
 extern bool iuConvert;		/* perform iso utf8 conversions automatically */
@@ -314,7 +323,9 @@ extern bool inputReadLine;
 extern bool curlAuthNegotiate;  /* try curl negotiate (SPNEGO) auth */
 extern bool listNA;		/* list nonascii chars */
 extern bool inInput;		/* reading line from standard in */
+extern bool inInitFunction; // executing the init function
 extern int fileSize;		/* when reading/writing files */
+extern int timer_sn; // timer sequence numbers
 extern char errorMsg[];		/* generated error message */
 extern int localAccount;	/* this is the smtp server for outgoing mail */
 extern char *mailDir;		/* move to this directory when fetching mail */
@@ -327,7 +338,8 @@ extern char *downDir;		/* the download directory */
 extern char *ebTempDir;		/* edbrowse temp, such as /tmp/.edbrowse */
 extern char *ebUserDir;		/* $ebTempDir/nnn user ID appended */
 extern char *dbarea, *dblogin, *dbpw;	/* to log into the database */
-extern bool fetchBlobColumns;
+extern bool fetchBlobColumns, ls_reverse;
+extern uchar ls_sort, dirWrite;
 extern bool caseInsensitive, searchStringsAll, searchWrap;
 extern bool allowRedirection;	/* from http code 301, or http refresh */
 extern bool sendReferrer;	/* in the http header */
@@ -340,10 +352,12 @@ extern bool pluginsOn;		/* plugins are active */
 extern bool showHiddenFiles;	/* during directory scan */
 extern bool showHover; // messages that appear when you hover
 extern bool doColors;
-extern int context;		/* which session (buffer) are we in? */
+extern int context;		// which session (buffer) are we in?
 extern pst linePending;
 extern char *changeFileName;
-extern char *addressFile;	/* your address book */
+extern char *addressFile;	// your address book
+extern char *emojiFile;	// your library of emojis
+extern int redirect_count;
 extern char *serverData;
 extern int serverDataLen;
 extern char *breakLineResult;
@@ -387,14 +401,18 @@ struct ebFrame {
 	struct ebWindow *owner;
 	struct htmlTag *frametag;
 	int gsn; // global sequence number
-	char *fileName;		/* name of file or url */
+	char *fileName;		// name of file or url
 	char *firstURL;		// before http redirection
 	char *hbase; /* base for href references */
+	const char *charset;		// charset for this page
 	bool render1; // rendered via protocol or urlmatch
 	bool render2; // rendered via suffix
 	bool render1b;
 	bool baseset; // <base> tag has been seen
 	bool uriEncoded; // filename is url encoded
+	bool jslink; // linke to javascript
+	bool browseMode;
+	short jtmin;
 	char *dw;		/* document.write string */
 	int dw_l;		/* length of the above */
 // document.writes go under the body.
@@ -407,7 +425,6 @@ struct ebFrame {
 	jsobjtype docobj;	/* window.document */
 	const struct MIMETYPE *mt;
 	void *cssmaster;
-	short jtmin;
 };
 
 typedef struct ebFrame Frame;
@@ -426,10 +443,12 @@ struct ebWindow {
  * Use the back command to pop the stack.
  * The back command follows this link, which is 0 if you are at the top. */
 	struct ebWindow *prev;
+	int sno; // session number
 /* This is right out of the source for ed.  Current and last line numbers. */
 	int dot, dol;
 /* remember dot and dol for the raw text, when in browse mode */
 	int r_dot, r_dol;
+	int f_dot; // foreward dot, in case we browse again
 	struct ebFrame f0; /* first frame */
 	struct ebFrame *jdb_frame; // if in jdb mode
 	char *referrer; // another web page that brought this one to life
@@ -452,11 +471,15 @@ struct ebWindow {
  * and used thereafter for hyperlinks, fill-out forms, etc. */
 	struct htmlTag **tags;
 	int numTags, allocTags, deadTags;
+int ehsn;
 	struct htmlTag *scriptlist, *inputlist, *optlist, *linklist;
+	struct htmlTag *framelist;
 	bool mustrender:1;
 	bool sank:1; /* jSyncup has been run */
 	bool lhs_yes:1;
 	bool rhs_yes:1;
+	bool lhs_bang:1;
+	bool lhs_ci:1;
 	bool binMode:1;		/* binary file */
 	bool nlMode:1;		/* newline at the end */
 	bool rnlMode:1;
@@ -473,11 +496,12 @@ struct ebWindow {
 	bool quitMode:1;	/* you can quit this buffer any time */
 	bool dirMode:1;		/* directory mode */
 	bool undoable:1;	/* undo is possible */
-	bool sqlMode:1;		/* accessing a table */
+	bool sqlMode:1;		// accessing a table
 	struct DBTABLE *table;	/* if in sqlMode */
 	time_t nextrender;
 };
-extern struct ebWindow *cw;	/* current window */
+typedef struct ebWindow Window;
+extern Window *cw;	/* current window */
 #define foregroundWindow (cw == sessionList[context].lw)
 
 /* quickly grab a tag from the current window via its sequence number:
@@ -485,7 +509,7 @@ extern struct ebWindow *cw;	/* current window */
 #define tagList (cw->tags)
 
 /* js is running in the current session */
-#define isJSAlive (cf->cx && allowJS)
+#define isJSAlive (cf->jslink && allowJS)
 
 /*********************************************************************
 Temporary cap on the number of lines, so the integer index into cw->map
@@ -499,7 +523,7 @@ If ints are larger then I don't even use this constant.
 
 /* An edit session */
 struct ebSession {
-	struct ebWindow *fw, *lw;	/* first window, last window */
+	Window *fw, *lw;	/* first window, last window */
 };
 extern struct ebSession sessionList[];
 extern struct ebSession *cs;	/* current session */
@@ -535,9 +559,8 @@ struct htmlTag {
 	struct htmlTag *same; // same action
 	struct ebFrame *f0; /* frame that owns this tag */
 	struct ebFrame *f1; /* subordinate frame if this is a <frame> tag */
-	jsobjtype jv;		/* corresponding javascript variable */
-	jsobjtype style; // style object
-	int seqno;
+	jsobjtype jv;		// javascript value
+	int seqno; // tag sequence number in this window
 	int gsn; // global sequence number, for rooting
 	char *js_file;
 	int js_ln;			/* line number of javascript */
@@ -560,6 +583,7 @@ struct htmlTag {
 	bool dead:1; // removed by garbage collection
 	bool contracted:1; /* frame is contracted */
 	bool multiple:1;
+	bool required:1;
 	bool async:1; // asynchronous script
 	bool intimer:1; // asynchronous script in timer
 	bool inxhr:1; // script is really an xhr
@@ -573,8 +597,10 @@ struct htmlTag {
 	bool post:1;		/* post, rather than get */
 	bool javapost:1;	// post by calling javascript
 	bool jslink:1;	// linked to a js object
-	bool mime:1;		/* encode as mime, rather than url encode */
-	bool bymail:1;		/* send by mail, rather than http */
+	bool expf:1; // we tried to expand this frame
+	bool mime:1;		// encode as mime, rather than url encode
+	bool plain:1;		// do not encode, rather than url encode
+	bool bymail:1;		// send by mail, rather than http
 	bool submitted:1;
 	bool onclick:1;
 	bool onchange:1;
@@ -586,6 +612,8 @@ struct htmlTag {
 	bool visited:1;
 	bool masked:1;
 	bool iscolor:1;
+	bool ur:1;		// row unfolded, only for trf
+	bool inur:1;		// in ur command
 	char subsup;		/* span turned into sup or sub */
 	uchar itype;		// input type =
 	uchar itype_minor;
@@ -599,6 +627,7 @@ struct htmlTag {
 // class is reserved word in c++, so use jclass for javascript class
 	char *name, *id, *jclass, *nodeName, *value, *href;
 	const char *rvalue; /* for reset */
+	char *custom_h; // http headers for a frame or xhr tag
 	char *innerHTML; /* the html string under this tag */
 	int inner;		/* for inner html */
 	int highspec; // specificity of a selector that matches this node
@@ -608,20 +637,21 @@ typedef struct htmlTag Tag;
 
 /* htmlTag.action */
 enum {
-	TAGACT_HTML, TAGACT_A, TAGACT_INPUT, TAGACT_TITLE, TAGACT_TA,
-	TAGACT_BUTTON, TAGACT_SELECT, TAGACT_OPTION, TAGACT_LABEL,
+	TAGACT_DOCTYPE, TAGACT_HTML, TAGACT_A, TAGACT_INPUT, TAGACT_TITLE,
+	TAGACT_TA, TAGACT_SELECT, TAGACT_OPTION, TAGACT_OPTG,
 	TAGACT_NOP, TAGACT_JS, TAGACT_H, TAGACT_SUB, TAGACT_SUP, TAGACT_OVB,
-	TAGACT_OL, TAGACT_UL, TAGACT_DL, TAGACT_TEXT,
+	TAGACT_OL, TAGACT_UL, TAGACT_DL, TAGACT_TEXT, TAGACT_BQ,
 	TAGACT_BODY, TAGACT_HEAD, TAGACT_DOC, TAGACT_FRAG, TAGACT_COMMENT,
 	TAGACT_MUSIC, TAGACT_IMAGE, TAGACT_BR, TAGACT_IBR, TAGACT_P,
-	TAGACT_BASE, TAGACT_META, TAGACT_LINK, TAGACT_PRE,
+	TAGACT_BASE, TAGACT_META, TAGACT_LINK, TAGACT_PRE, TAGACT_LABEL,
 	TAGACT_TBODY, TAGACT_THEAD, TAGACT_TFOOT,
 	TAGACT_DT, TAGACT_DD, TAGACT_LI, TAGACT_TABLE, TAGACT_TR, TAGACT_TD,
 	TAGACT_DIV, TAGACT_SPAN, TAGACT_HR, TAGACT_OBJECT, TAGACT_FOOTER,
 	TAGACT_HEADER, // <header> tag, not the same as <head> tag
-	TAGACT_FORM, TAGACT_FRAME, TAGACT_STYLE,
+	TAGACT_FORM, TAGACT_FRAME, TAGACT_STYLE, TAGACT_SVG,
+	TAGACT_CANVAS,
 	TAGACT_MAP, TAGACT_AREA, TAGACT_SCRIPT, TAGACT_NOSCRIPT, TAGACT_EMBED,
-	TAGACT_OBJ, TAGACT_UNKNOWN,
+	TAGACT_OBJ, TAGACT_DATAL, TAGACT_UNKNOWN,
 };
 
 /* htmlTag.itype */
