@@ -10,29 +10,25 @@ we need to indicate that all the other functions are straight C.
 extern "C" {
 #endif
 
-/* sourcefile=auth.c */
-bool getUserPass(const char *url, char *creds, bool find_proxy) ;
-bool getUserPassRealm(const char *url, char *creds, const char *realm);
-// Add authorization entries only in the foreground, but it's an
-// atomic operation, so the previous routines can run from other threads.
-bool addWebAuthorization(const char *url, const char *credentials, bool proxy, const char *realm);
-
 /* sourcefile=buffers.c */
+void undoSpecialClear(void);
 void removeHiddenNumbers(pst p, uchar terminate);
-pst fetchLine(int n, int show) ;
-void displayLine(int n) ;
-void initializeReadline(void) ;
-pst inputLine(void) ;
+pst fetchLine(int n, int show);
+void displayLine(int n);
+void initializeReadline(void);
+pst inputLine(void);
 bool cxCompare(int cx) ;
-bool cxActive(int cx) ;
+bool cxActive(int cx, bool error);
 bool cxQuit(int cx, int action) ;
 void cxSwitch(int cx, bool interactive) ;
 bool addTextToBuffer(const pst inbuf, int length, int destl, bool showtrail) ;
 void delText(int start, int end) ;
-bool readFileArgv(const char *filename, int fromframe);
-bool unfoldBufferW(const struct ebWindow *w, bool cr, char **data, int *len) ;
-bool unfoldBuffer(int cx, bool cr, char **data, int *len) ;
-bool runCommand(const char *line) ;
+bool readFileArgv(const char *filename, int fromframe, const char *orig_head);
+Tag *line2frame(int ln);
+bool unfoldBufferW(const struct ebWindow *w, bool cr, char **data, int *len);
+bool unfoldBuffer(int cx, bool cr, char **data, int *len);
+bool jump2anchor(const Tag *jumptag, const char *newhash);
+bool runCommand(const char *line);
 bool edbrowseCommand(const char *line, bool script) ;
 int sideBuffer(int cx, const char *text, int textlen, const char *bufname);
 void freeEmptySideBuffer(int n);
@@ -41,20 +37,6 @@ bool locateTagInBuffer(int tagno, int *ln_p, char **p_p, char **s_p, char **t_p)
 char *getFieldFromBuffer(int tagno);
 int fieldIsChecked(int tagno);
 
-/* sourcefile=cookies.c */
-bool domainSecurityCheck(const char *server, const char *domain) ;
-bool receiveCookie(const char *url, const char *str) ;
-void cookiesFromJar(void) ;
-void sendCookies(char **s, int *l, const char *url, bool issecure) ;
-void mergeCookies(void);
-
-/* sourcefile=cache.c */
-void setupEdbrowseCache(void);
-void clearCache(void) ;
-bool fetchCache(const char * url, const char *etag, time_t modtime, char **data, int *data_len) ;
-void storeCache(const char *url, const char *etag, time_t modtime, const char *data, int datalen) ;
-bool presentInCache(const char *url) ;
-
 /* sourcefile=dbodbc.c (and others) */
 bool fetchForeign(char *tname) ;
 
@@ -73,48 +55,8 @@ bool sqlAddRows(int ln) ;
 bool ebConnect(void) ;
 int goSelect(int *startLine, char **rbuf) ;
 
-/* sourcefile=ebjs.c */
-void javaSetsTagVar(jsobjtype v, const char *newtext);
-void dwStart(void);
-void createJavaContext(void) ;
-void freeJavaContext(Frame *f) ;
-char *jsRunScriptResult(const Frame *f, jsobjtype obj, const char *str, const char *filename, int lineno) ;
-void jsRunScript(const Frame *f, jsobjtype obj, const char *str, const char *filename, int lineno) ;
-void jsRunData(const Frame *f, jsobjtype obj, const char *filename, int lineno);
-enum ej_proptype typeof_property(const Frame *f, jsobjtype obj, const char *name) ;
-#define handlerPresent(f, obj, name) (typeof_property(f, obj, name) == EJ_PROP_FUNCTION)
-bool has_property(const Frame *f, jsobjtype obj, const char *name) ;
-void delete_property(const Frame *f, jsobjtype obj, const char *name) ;
-char *get_property_string(const Frame *f, jsobjtype obj, const char *name) ;
-int get_property_number(const Frame *f, jsobjtype obj, const char *name) ;
-double get_property_float(const Frame *f, jsobjtype obj, const char *name) ;
-bool get_property_bool(const Frame *f, jsobjtype obj, const char *name) ;
-jsobjtype get_property_object(const Frame *f, jsobjtype parent, const char *name) ;
-jsobjtype get_property_function(const Frame *f, jsobjtype parent, const char *name);
-jsobjtype get_array_element_object(const Frame *f, jsobjtype obj, int idx) ;
-int set_property_string(const Frame *f, jsobjtype obj, const char *name, const char *value) ;
-int set_property_number(const Frame *f, jsobjtype obj, const char *name, int n) ;
-int set_property_float(const Frame *f, jsobjtype obj, const char *name, double n) ;
-int set_property_bool(const Frame *f, jsobjtype obj, const char *name, bool n) ;
-int set_property_object(const Frame *f, jsobjtype parent, const char *name, jsobjtype child) ;
-jsobjtype instantiate_array(const Frame *f, jsobjtype parent, const char *name) ;
-int set_array_element_object(const Frame *f, jsobjtype array, int idx, jsobjtype child) ;
-jsobjtype instantiate_array_element(const Frame *f, jsobjtype array, int idx, const char *classname) ;
-jsobjtype instantiate(const Frame *f, jsobjtype parent, const char *name, const char *classname) ;
-int set_property_function(const Frame *f, jsobjtype parent, const char *name, const char *body) ;
-int get_arraylength(const Frame *f, jsobjtype a);
-void setupJavaDom(void) ;
-char *get_property_url(const Frame *f, jsobjtype owner, bool action) ;
-void rebuildSelectors(void);
-bool run_function_bool(const Frame *f, jsobjtype obj, const char *name);
-jsobjtype create_event(const Frame *f, jsobjtype parent, const char *evname);
-void unlink_event(const Frame *f, jsobjtype parent);
-bool run_event_bool(const Frame *f, jsobjtype obj, const char *pname, const char *evname);
-int run_function_onearg(const Frame *f, jsobjtype obj, const char *name, jsobjtype o);
-void run_function_onestring(const Frame *f, jsobjtype parent, const char *name, const char *s);
-void set_basehref(const char *b);
-
 /* sourcefile=fetchmail.c */
+bool setEnvelopeFormat(const char *s);
 int fetchMail(int account);
 int fetchAllMail(void);
 void scanMail(void);
@@ -137,6 +79,8 @@ bool looksBinary(const unsigned char *buf, int buflen);
 void looks_8859_utf8(const uchar *buf, int buflen, bool * iso_p, bool * utf8_p);
 void iso2utf(const uchar *inbuf, int inbuflen, uchar **outbuf_p, int *outbuflen_p);
 void utf2iso(const uchar *inbuf, int inbuflen, uchar **outbuf_p, int *outbuflen_p);
+void utf2iso1(char *s, size_t *lenp);
+char *iso12utf(const char *t1, const char *t2, int *lenp);
 void utfHigh(const char *inbuf, int inbuflen, char **outbuf_p, int *outbuflen_p, bool inutf8, bool out32, bool outbig);
 char *uni2utf8(unsigned int unichar);
 void utfLow(const char *inbuf, int inbuflen, char **outbuf_p, int *outbuflen_p, int bom);
@@ -145,39 +89,53 @@ char *base64Encode(const char *inbuf, int inlen, bool lines);
 uchar base64Bits(char c);
 int base64Decode(char *start, char **end);
 void iuReformat(const char *inbuf, int inbuflen, char **outbuf_p, int *outbuflen_p);
+char *makeDosNewlines(char *p);
 bool parseDataURI(const char *uri, char **mediatype, char **data, int *data_l);
-uchar fromHex(char d, char e) ;
+uchar fromHex(char d, char e);
 char *closeColor(const char *s);
+void clearEmojis(void);
+void loadEmojis(void);
+char *selectEmoji(const char *p, int len);
 
 /* sourcefile=html.c */
+void dwStart(void);
+void set_basehref(const char *b);
 bool tagHandler(int seqno, const char *name);
+void jSyncup(bool fromtimer, const Tag *active);
 void jSideEffects(void);
-void jSyncup(bool fromtimer);
 void jClearSync(void);
-void htmlMetaHelper(Tag *t);
+void htmlMetaHelper(const Tag *t);
 void prepareScript(Tag *t);
+bool isRooted(const Tag *t);
 void runScriptsPending(bool startbrowse);
-void preFormatCheck(int tagno, bool * pretag, bool * slash) ;
-char *htmlParse(char *buf, int remote) ;
-bool htmlTest(void) ;
-void infShow(int tagno, const char *search) ;
-void itext(void);
+void preFormatCheck(int tagno, bool * pretag, bool * slash);
+char *htmlParse(char *buf, int remote);
+bool htmlTest(void);
+void infShow(int tagno, const char *search);
+bool itext(int d);
 bool infReplace(int tagno, const char *newtext, bool notify);
 bool infPush(int tagno, char **post_string);
-Tag *tagFromJavaVar(jsobjtype v);
-void javaSubmitsForm(jsobjtype v, bool reset);
-bool bubble_event(const Tag *t, const char *name);
+void domSetsTagValue(Tag *t, const char *newtext);
+void domSubmitsForm(Tag *t, bool reset);
+bool charInOptions(char c);
+void charFixOptions(char c);
+bool charInFiles(char c);
+void charFixFiles(char c);
+const Tag *gebi_c(const Tag *t, const char *id, bool lookname);
 void runningError(int msg, ...);
-void rerender(bool notify);
+void rerender(int notify);
 void delTags(int startRange, int endRange);
 void runOnload(void);
-void javaSetsTimeout(int n, const char *jsrc, jsobjtype to, bool isInterval);
+const char *tack_fn(const char *e);
+void domSetsTimeout(int n, const char *jsrc, const char *backlink, bool isInterval);
 void scriptSetsTimeout( Tag *t);
 bool timerWait(int *delay_sec, int *delay_ms);
-void delTimers(Frame *f);
+void delTimers(const Frame *f);
 void runTimer(void);
-void javaOpensWindow(const char *href, const char *name) ;
-void javaSetsLinkage(bool after, char type, jsobjtype p, const char *rest);
+void showTimers(void);
+void domOpensWindow(const char *href, const char *name);
+struct htmlTag *line2tr(int ln);
+bool showHeaders(int ln);
 
 /* sourcefile=html-tidy.c */
 void html2nodes(const char *htmltext, bool startpage);
@@ -185,29 +143,34 @@ void html2nodes(const char *htmltext, bool startpage);
 /* sourcefile=decorate.c */
 void traverseAll(int start);
 const char *attribVal(const Tag *t, const char *name);
+bool attribPresent(const Tag *t, const char *name);
+void setTagAttr(Tag *t, const char *name, char *val);
 Tag *findOpenTag(Tag *t, int action);
 Tag *findOpenList(Tag *t);
 void formControl(Tag *t, bool namecheck);
 void htmlInputHelper(Tag *t);
-char *displayOptions(const Tag *sel) ;
+char *displayOptions(const Tag *sel);
 void prerender(int start);
-jsobjtype instantiate_url(const Frame *f, jsobjtype parent, const char *name, const char *url) ;
 char *render(int start);
+const char *fakePropName(void);
 void decorate(int start);
-void freeTags(struct ebWindow *w) ;
-Tag *newTag(const Frame *f, const char *tagname) ;
+void freeTags(struct ebWindow *w);
+Tag *newTag(const Frame *f, const char *tagname);
 void initTagArray(void);
 void tag_gc(void);
 void tag_gc(void);
 void htmlNodesIntoTree(int start, Tag *attach);
 void underKill(Tag *t);
-void killTag(Tag *t);
-void html_from_setter( jsobjtype innerParent, const char *h);
+void html_from_setter( Tag *innerParent, const char *h);
+void debugGenerated(const char *h);
+bool frameFromContext(jsobjtype cx);
 
 /* sourcefile=http.c */
-size_t eb_curl_callback(char *incoming, size_t size, size_t nitems, struct i_get *g) ;
-time_t parseHeaderDate(const char *date) ;
-bool parseRefresh(char *ref, int *delay_p) ;
+void eb_curl_global_init(void);
+void eb_curl_global_cleanup(void);
+size_t eb_curl_callback(char *incoming, size_t size, size_t nitems, struct i_get *g);
+time_t parseHeaderDate(const char *date);
+bool parseRefresh(char *ref, int *delay_p);
 bool shortRefreshDelay(const char *r, int d);
 bool httpConnect(struct i_get *g);
 void *httpConnectBack1(void *ptr);
@@ -220,15 +183,10 @@ int ebcurl_debug_handler(CURL * handle, curl_infotype info_desc, char *data, siz
 int bg_jobs(bool iponly);
 CURLcode setCurlURL(CURL * h, const char *url);
 bool frameExpand(bool expand, int ln1, int ln2);
-int frameExpandLine(int ln, jsobjtype fo);
-Tag *line2frame(int ln);
+int frameExpandLine(int ln, Tag *t);
 bool reexpandFrame(void);
-bool frameSecurityFile(const char *thisfile);
-void unframe(jsobjtype fobj, jsobjtype newdoc);
-void unframe2(jsobjtype fobj);
 
 /* sourcefile=main.c */
-void eb_curl_global_init(void);
 void ebClose(int n);
 void setDataSource(char *v);
 bool javaOK(const char *url);
@@ -236,10 +194,10 @@ bool mustVerifyHost(const char *url);
 const char *findProxyForURL(const char *url);
 const char *findAgentForURL(const char *url);
 const char *mailRedirect(const char *to, const char *from, const char *reply, const char *subj);
-bool runEbFunction(const char *line);
+int runEbFunction(const char *line);
 struct DBTABLE *findTableDescriptor(const char *sn);
 struct DBTABLE *newTableDescriptor(const char *name);
-void readConfigFile(void);
+bool readConfigFile(void);
 const char *fetchReplace(const char *u);
 
 /* sourcefile=plugin.c */
@@ -281,9 +239,9 @@ bool helpUtility(void);
 // which are still threadsafe the way we use them, e.g. only calling
 // them from the foreground thread, or ? which is not safe.
 // I'd like to do this markup for the other files as time permits.
-void *allocMem(size_t n) ;
-void *allocZeroMem(size_t n) ;
-void *reallocMem(void *p, size_t n) ;
+void *allocMem(size_t n);
+void *allocZeroMem(size_t n);
+void *reallocMem(void *p, size_t n);
 char *allocString(size_t n) ;
 char *allocZeroString(size_t n) ;
 char *reallocString(void *p, size_t n) ;
@@ -314,7 +272,7 @@ int stringIsNum(const char *s) ;
 bool stringIsDate(const char *s) ;
 bool stringIsFloat(const char *s, double *dp) ;
 bool memEqualCI(const char *s, const char *t, int len) ;
-char *strstrCI(const char *base, const char *search) ;
+const char *stringInBufLine(const char *s, const char *t);
 bool stringEqual(const char *s, const char *t) ;
 bool stringEqualCI(const char *s, const char *t) ;
 int stringInList(const char *const *list, const char *s) ;
@@ -353,6 +311,8 @@ char *conciseTime(time_t t); //?
 bool lsattrChars(const char *buf, char *dest);
 char *lsattr(const char *path, const char *flags); //?
 void ttySaveSettings(void) ; //@
+void ttyRestoreSettings(void);
+void ttyRaw(int charcount, int timeout, bool isecho);
 void ttySetEcho(bool enable_echo);
 #ifndef _INC_CONIO
 int getche(void) ; //@
@@ -362,17 +322,18 @@ char getLetter(const char *s) ;
 char *getFileName(int msg, const char *defname, bool isnew, bool ws);
 int shellProtectLength(const char *s);
 void shellProtect(char *t, const char *s);
-const char *nextScanFile(const char *base); //?
-bool sortedDirList(const char *dir, struct lineMap **map_p, int *count_p, int othersort, bool reverse) ; //?
-bool envFile(const char *line, const char **expanded); //?
-bool envFileDown(const char *line, const char **expanded) ; //?
+const char *nextScanFile(const char *base); //@
+bool sortedDirList(const char *dir, struct lineMap **map_p, int *count_p, int othersort, bool reverse) ; //@
+bool envFile(const char *line, const char **expanded); //@
+char *envFileAlloc(const char *line); //@
+bool envFileDown(const char *line, const char **expanded) ; //@
 FILE *efopen(const char *name, const char *mode) ;
 int eopen(const char *name, int mode, int perms) ;
 void appendFile(const char *fname, const char *message, ...) ;
 void appendFileNF(const char *filename, const char *msg) ;
 int eb_system(const char *cmd, bool print_on_success);
 
-/* sourcefile=url.c */
+/* sourcefile=isup.c */
 void unpercentURL(char *url) ;
 void unpercentString(char *s) ;
 char *percentURL(const char *start, const char *end);
@@ -403,53 +364,96 @@ char *encodePostData(const char *s, const char *keep_chars) ;
 char *decodePostData(const char *data, const char *name, int seqno) ;
 void decodeMailURL(const char *url, char **addr_p, char **subj_p, char **body_p) ;
 bool patternMatchURL(const char *url, const char *pattern);
+bool frameSecurityFile(const char *thisfile);
+bool receiveCookie(const char *url, const char *str) ;
+void cookiesFromJar(void) ;
+bool isInDomain(const char *d, const char *s);
+void sendCookies(char **s, int *l, const char *url, bool issecure) ;
+void mergeCookies(void);
+void setupEdbrowseCache(void);
+void clearCache(void) ;
+bool fetchCache(const char * url, const char *etag, time_t modtime, char **data, int *data_len) ;
+bool presentInCache(const char *url) ;
+void storeCache(const char *url, const char *etag, time_t modtime, const char *data, int datalen) ;
+bool getUserPass(const char *url, char *creds, bool find_proxy) ;
+bool getUserPassRealm(const char *url, char *creds, const char *realm);
+// Add authorization entries only in the foreground, but it's an
+// atomic operation, so the previous routines can run from other threads.
+bool addWebAuthorization(const char *url, const char *credentials, bool proxy, const char *realm);
 
 /* sourcefile=css.c */
 void writeShortCache(void);
 bool matchMedia(char *t);
-void cssDocLoad(jsobjtype thisobj, char *s, bool pageload);
+void cssDocLoad(int frameNumber, char *s, bool pageload);
 void cssFree(Frame *f);
-jsobjtype querySelectorAll(const char *selstring, jsobjtype topobj);
-jsobjtype querySelector(const char *selstring, jsobjtype topobj);
-bool querySelector0(const char *selstring, jsobjtype topobj);
-void cssApply(jsobjtype thisobj, jsobjtype node, jsobjtype destination);
-void cssText(jsobjtype node, const char *rulestring);
+Tag **querySelectorAll(const char *selstring, Tag *top);
+Tag *querySelector(const char *selstring, Tag *top);
+bool querySelector0(const char *selstring, Tag *top);
+void cssApply(int frameNumber, Tag *t, int pe);
+void cssText(const char *rulestring);
+
+/* sourcefile=jseng-* */
+void disconnectTagObject(Tag *t);
+void reconnectTagObject(Tag *t);
+bool has_property_t(const Tag *t, const char *name);
+bool has_property_win(const Frame *f, const char *name) ;
+void set_property_object_t(const Tag *t, const char *name, const Tag *t2);
+bool run_function_bool_t(const Tag *t, const char *name);
+bool run_function_bool_win(const Frame *f, const char *name);
+void forceFrameExpand(Tag *t);
+void my_ExecutePendingJobs(void);
+void my_ExecutePendingMessages(void);
+void my_ExecutePendingMessagePorts(void);
+void delPendings(const Frame *f);
+void js_main(void);
+void createJSContext(Frame *f);
+void freeJSContext(Frame *f);
+void run_ontimer(const Frame *f, const char *backlink);
+int run_function_onearg_t(const Tag *t, const char *name, const Tag *t2);
+int run_function_onearg_win(const Frame *f, const char *name, const Tag *t2);
+int run_function_onearg_doc(const Frame *f, const char *name, const Tag *t2);
+void run_function_onestring_t(const Tag *t, const char *name, const char *s);
+void run_function_onestring_win(const Frame *f, const char *name, const char *s);
+void jsRunData(const Tag *t, const char *filename, int lineno);
+bool run_event_t(const Tag *t, const char *pname, const char *evname);
+bool run_event_win(const Frame *f, const char *pname, const char *evname);
+bool run_event_doc(const Frame *f, const char *pname, const char *evname);
+bool bubble_event_t(const Tag *t, const char *name);
+void set_property_bool_win(const Frame *f, const char *name, bool v);
+char *get_property_url_t(const Tag *t, bool action);
+char *get_style_string_t(const Tag *t, const char *name);
+void delete_property_t(const Tag *t, const char *name);
+void delete_property_win(const Frame *f, const char *name);
+void delete_property_doc(const Frame *f, const char *name);
+bool get_property_bool_t(const Tag *t, const char *name);
+enum ej_proptype typeof_property_t(const Tag *t, const char *name);
+int get_property_number_t(const Tag *t, const char *name);
+char * get_property_string_t(const Tag *t, const char *name);
+char *get_dataset_string_t(const Tag *t, const char *name);
+void set_property_bool_t(const Tag *t, const char *name, bool v);
+void set_property_number_t(const Tag *t, const char *name, int v);
+void set_property_string_t(const Tag *t, const char *name, const char * v);
+void set_dataset_string_t(const Tag *t, const char *name, const char * v);
+void set_property_bool_win(const Frame *f, const char *name, bool v);
+void set_property_string_win(const Frame *f, const char *name, const char *v);
+void set_property_object_doc(const Frame *f, const char *name, const Tag *t2);
+void set_property_string_doc(const Frame *f, const char *name, const char *v);
+void jsRunScriptWin(const char *str, const char *filename, 		 int lineno);
+void jsRunScript_t(const Tag *t, const char *str, const char *filename, 		 int lineno);
+char *jsRunScriptWinResult(const char *str, const char *filename, int lineno) ;
+void establish_js_option(Tag *t, Tag *sel);
+void establish_js_textnode(Tag *t, const char *fpn);
+void domLink(Tag *t, const char *classname, const char *href, const char *list, const Tag *owntag, int extra);
+void rebuildSelectors(void);
+bool has_gcs(const char *name);
+enum ej_proptype typeof_gcs(const char *name);
+int get_gcs_number(const char *name);
+void set_gcs_number(const char *name, int n);
+void set_gcs_bool(const char *name, bool v);
+void set_gcs_string(const char *name, const char *s);
+void jsClose(void);
+void set_location_hash(const char *h);
 
 #ifdef __cplusplus
 }
 #endif
-
-/* sourcefile=jseng-duk.c */
-void connectTagObject(Tag *t, jsobjtype p);
-void disconnectTagObject(Tag *t);
-int js_main(void);
-// the native versions of the api functions in ebjs.c
-void createJavaContext_0(Frame *f);
-void freeJavaContext_0(jsobjtype cx);
-enum ej_proptype typeof_property_0(jsobjtype cx, jsobjtype obj, const char *name) ;
-bool has_property_0(jsobjtype cx, jsobjtype obj, const char *name) ;
-void delete_property_0(jsobjtype cx, jsobjtype obj, const char *name) ;
-char *get_property_string_0(jsobjtype cx, jsobjtype obj, const char *name) ;
-int get_property_number_0(jsobjtype cx, jsobjtype parent, const char *name) ;
-double get_property_float_0(jsobjtype cx, jsobjtype parent, const char *name) ;
-bool get_property_bool_0(jsobjtype cx, jsobjtype parent, const char *name) ;
-jsobjtype get_property_object_0(jsobjtype cx, jsobjtype parent, const char *name) ;
-jsobjtype get_property_function_0(jsobjtype cx, jsobjtype parent, const char *name) ;
-jsobjtype get_array_element_object_0(jsobjtype cx, jsobjtype obj, int idx) ;
-int set_property_string_0(jsobjtype cx, jsobjtype obj, const char *name, const char *value) ;
-int set_property_number_0(jsobjtype cx, jsobjtype obj, const char *name, int n) ;
-int set_property_float_0(jsobjtype cx, jsobjtype obj, const char *name, double n) ;
-int set_property_bool_0(jsobjtype cx, jsobjtype obj, const char *name, bool n) ;
-int set_property_object_0(jsobjtype cx, jsobjtype parent, const char *name, jsobjtype child) ;
-jsobjtype instantiate_array_0(jsobjtype cx, jsobjtype parent, const char *name) ;
-int set_array_element_object_0(jsobjtype cx, jsobjtype array, int idx, jsobjtype child) ;
-jsobjtype instantiate_array_element_0(jsobjtype cx, jsobjtype array, int idx, const char *classname) ;
-jsobjtype instantiate_0(jsobjtype cx, jsobjtype parent, const char *name, const char *classname) ;
-int set_property_function_0(jsobjtype cx, jsobjtype parent, const char *name, const char *body) ;
-int get_arraylength_0(jsobjtype cx, jsobjtype a);
-bool run_function_bool_0(jsobjtype cx, jsobjtype obj, const char *name);
-int run_function_onearg_0(jsobjtype cx, jsobjtype obj, const char *name, jsobjtype o);
-void run_function_onestring_0(jsobjtype cx, jsobjtype parent, const char *name, const char *s);
-char *run_script_0(jsobjtype cx, const char *s);
-void run_data_0(jsobjtype cx, jsobjtype o);
-
