@@ -11,7 +11,7 @@
 #include "vsprtf.h"
 #endif
 
-#define MHLINE 400		/* length of a mail header line */
+#define MHLINE 512		// length of a mail header line
 /* headers and other information about an email */
 struct MHINFO {
 	struct MHINFO *next, *prev;
@@ -58,7 +58,7 @@ static void freeMailInfo(struct MHINFO *w)
 	nzFree(w->tolist);
 	nzFree(w->cclist);
 	nzFree(w);
-}				/* freeMailInfo */
+}
 
 static bool ignoreImages;
 
@@ -122,7 +122,7 @@ static void writeAttachment(struct MHINFO *w)
 			close(fh);
 		}
 	}
-}				/* writeAttachment */
+}
 
 static void writeAttachments(struct MHINFO *w)
 {
@@ -133,15 +133,55 @@ static void writeAttachments(struct MHINFO *w)
 		foreach(v, w->components)
 		    writeAttachments(v);
 	}
-}				/* writeAttachments */
+}
 
 /* string to hold the returned data from the mail server */
 static char *mailstring;
 static int mailstring_l;
 static char *mailbox_url, *message_url;
 
-static int imapfetch = 100;
+int imapfetch = 100;
 static bool earliest;
+static const char envelopeFormatChars[] = "tfsdz";
+// to from subject date size
+static char envelopeFormat[6] = {'f', 's'}; // default is from subject
+static char envelopeFormatDef[6] = { 'f', 's'};
+
+bool setEnvelopeFormat(const char *s)
+{
+	char c;
+	bool something = false;
+	int i, j;
+	char count[6];
+
+// check
+	for(j=0; (c = s[j]); ++j) {
+		if(isspace(c))
+			continue;
+		something = true;
+		if(!strchr(envelopeFormatChars, c))
+			return false;
+	}
+
+// e command alone resurrects the default
+	if(!something) {
+		strcpy(envelopeFormat, envelopeFormatDef);
+		return true;
+	}
+
+	memset(count, 0, sizeof(count));
+	for(i=0; (c = *s); ++s) {
+		if(isspace(c))
+			continue;
+		j = strchr(envelopeFormatChars, c) - envelopeFormatChars;
+		if(count[j])
+			continue;
+		count[j] = 1;
+		envelopeFormat[i++] = c;
+	}
+	envelopeFormat[i] = 0;
+	return true;
+}
 
 static void setLimit(const char *t)
 {
@@ -158,7 +198,7 @@ static void setLimit(const char *t)
 	if (imapfetch < 10)
 		imapfetch = 10;
 	i_printf(MSG_FetchN, imapfetch);
-}				/* setLimit */
+}
 
 /* mail message in a folder */
 struct MIF {
@@ -166,10 +206,11 @@ struct MIF {
 	int seqno;
 	int size;
 	char *cbase;		/* allocated string containing the following */
-	char *subject, *from, *reply;
-	char *refer;
+	char *subject, *from, *to, *reply;
+// references, principal recipient, carbon recipient
+	char *refer, *prec, *ccrec;
 	time_t sent;
-	bool seen, gone;
+	bool seen, gone, line2;
 };
 
 /* folders at the top of an imap system */
@@ -253,6 +294,13 @@ static void setFolders(CURL * handle)
 			*t = 0;
 			f->name = s;
 			s = t + 1;
+// \Noselect \UnMarked  not sure what this means but it derails,
+// we need to focus on Noselect
+			t = strchr(f->name, '\\');
+			if(t) {
+				while(t > f->name && t[-1] == ' ') --t;
+				*t = 0;
+			}
 /* the null folder at the top, like /, isn't really a folder. */
 			if (stringEqual(f->name, "Noselect"))
 				continue;
@@ -286,7 +334,7 @@ static void setFolders(CURL * handle)
 	f = topfolders;
 	for (i = 0; i < n_folders; ++i, ++f)
 		examineFolder(handle, f, true);
-}				/* setFolders */
+}
 
 static struct FOLDER *folderByName(char *line)
 {
@@ -304,7 +352,7 @@ static struct FOLDER *folderByName(char *line)
 
 	f = topfolders;
 	for (i = 0; i < n_folders; ++i, ++f)
-		if (strstrCI(f->path, line))
+		if (strcasestr(f->path, line))
 			++cnt, j = i;
 	if (cnt == 1)
 		return topfolders + j;
@@ -313,7 +361,7 @@ static struct FOLDER *folderByName(char *line)
 	else
 		i_printf(MSG_NoFolderMatch, line);
 	return 0;
-}				/* folderByName */
+}
 
 /* data block for the curl ccallback write function in http.c */
 static struct i_get callback_data;
@@ -322,6 +370,7 @@ static CURLcode getMailData(CURL * handle)
 {
 	static bool first_call = true;
 	CURLcode res;
+//  puts("get data");
 	callback_data.buffer = initString(&callback_data.length);
 	callback_data.move_capable = false;
 	res = curl_easy_perform(handle);
@@ -339,9 +388,12 @@ static CURLcode getMailData(CURL * handle)
 	return res;
 }
 
-/* imap emails come in through the headers, not the data.
- * No kidding! I don't understand it either.
- * This callback function doesn't use a data block, mailstring is assumed. */
+/*********************************************************************
+imap emails come in through the headers, not the data.
+No kidding! I don't understand it either.
+This callback function doesn't use a data block, mailstring is assumed.
+*********************************************************************/
+
 static size_t imap_header_callback(char *i, size_t size,
 				   size_t nitems, void *data)
 {
@@ -356,7 +408,7 @@ static size_t imap_header_callback(char *i, size_t size,
 		fflush(stdout);
 	}
 	return b;
-}				/* imap_header_callback */
+}
 
 /* after the email has been fetched via pop3 or imap */
 static void undosOneMessage(void)
@@ -375,7 +427,7 @@ static void undosOneMessage(void)
 	}
 	mailstring_l = k;
 	mailstring[k] = 0;
-}				/* undosOneMessage */
+}
 
 static char presentMail(void);
 static void envelopes(CURL * handle, struct FOLDER *f);
@@ -393,7 +445,7 @@ static void cleanFolder(struct FOLDER *f)
 	nzFree(f->mlist);
 	f->mlist = NULL;
 	f->nmsgs = f->nfetch = f->unread = 0;
-}				/* cleanFolder */
+}
 
 /* search through imap server for a particular string */
 /* Return true if the search ran successfully and found some messages. */
@@ -477,7 +529,7 @@ none:
 	f->mlist = allocZeroMem(sizeof(struct MIF) * f->nfetch);
 	mif = f->mlist;
 	u = t;
-	while (*u) {
+	while (*u && cnt) {
 		int seqno = strtol(u, &u, 10);
 		if (cnt <= f->nfetch) {
 			mif->seqno = seqno;
@@ -490,30 +542,50 @@ none:
 	envelopes(handle, f);
 
 	return true;
-}				/* imapSearch */
+}
 
 static void printEnvelope(const struct MIF *mif)
 {
 	char *envp;		// print the envelope concisely
 	char *envp_end;
 	int envp_l;
+	int i;
+	char c;
 	envp = initString(&envp_l);
 #if 0
 	if (!mif->seen)
 		stringAndChar(&envp, &envp_l, '*');
 #endif
-	stringAndString(&envp, &envp_l, mif->from);
-	stringAndString(&envp, &envp_l, ": ");
-	stringAndString(&envp, &envp_l, mif->subject);
-	if (mif->sent) {
-		stringAndChar(&envp, &envp_l, ' ');
-		stringAndString(&envp, &envp_l, conciseTime(mif->sent));
+	if(mif->line2 && debugLevel >= 3)
+		stringAndString(&envp, &envp_l, "~ ");
+
+	for (i = 0; (c = envelopeFormat[i]); ++i) {
+		if(i)
+			stringAndString(&envp, &envp_l, " | ");
+		switch(c) {
+		case 'f':
+			stringAndString(&envp, &envp_l, mif->from[0] ? mif->from : mif->reply);
+			break;
+		case 't':
+			stringAndString(&envp, &envp_l, mif->to[0] ? mif->to : mif->prec);
+			break;
+		case 's':
+			stringAndString(&envp, &envp_l, mif->subject);
+			break;
+		case 'd':
+			if (mif->sent)
+				stringAndString(&envp, &envp_l, conciseTime(mif->sent));
+			break;
+		case 'z':
+			stringAndString(&envp, &envp_l, conciseSize(mif->size));
+			break;
+		}
 	}
-	stringAndChar(&envp, &envp_l, ' ');
-	stringAndString(&envp, &envp_l, conciseSize(mif->size));
+
 	envp_end = envp + envp_l;
 	isoDecode(envp, &envp_end);
 	*envp_end = 0;
+
 // Resulting line could contain utf8, use the portable puts routine.
 	eb_puts(envp);
 	nzFree(envp);
@@ -596,7 +668,9 @@ abort:
 	return -1;
 }
 
+
 /* scan through the messages in a folder */
+static char postkey;
 static void scanFolder(CURL * handle, struct FOLDER *f)
 {
 	struct MIF *mif;
@@ -637,9 +711,10 @@ reaction:
 		printEnvelope(mif);
 action:
 		delflag = false;
+		postkey = 0;
 		printf("? ");
 		fflush(stdout);
-		key = getLetter("h?qvbfdslmn /");
+		key = getLetter("h?qvbefdslmnp gwWuUa/");
 		printf("\b\b\b");
 		fflush(stdout);
 		if (key == '?' || key == 'h') {
@@ -660,7 +735,7 @@ imap_done:
 			if (!fgets(inputline, sizeof(inputline), stdin))
 				goto imap_done;
 			if (!imapSearch(handle, f, inputline))
-				goto action;
+				goto reaction;
 			if (f->nmsgs > f->nfetch)
 				i_printf(MSG_ShowLast + earliest, f->nfetch, f->nmsgs);
 			else
@@ -668,18 +743,28 @@ imap_done:
 			goto showmessages;
 		}
 
-		if (key == ' ') {
+		if(strchr("wWuUa", key))
+			postkey = key, key = ' ';
+
+		if (key == ' ' || key == 'g') {
 /* download the email from the imap server */
 			sprintf(cust_cmd, "FETCH %d BODY[]", mif->seqno);
 			curl_easy_setopt(handle, CURLOPT_CUSTOMREQUEST,
 					 cust_cmd);
 			mailstring = initString(&mailstring_l);
-/* I wanted to turn the write function off here, because we don't need it,
- * but turning it off and leaving HEADERFUNCTION on causes a seg fault,
- * I have no idea why.
- * curl_easy_setopt(handle, CURLOPT_WRITEFUNCTION, NULL);
- * curl_easy_setopt(handle, CURLOPT_WRITEDATA, NULL);
- */
+
+/*********************************************************************
+I wanted to turn the write function off here, because we don't need it.
+	curl_easy_setopt(handle, CURLOPT_WRITEFUNCTION, NULL);
+	curl_easy_setopt(handle, CURLOPT_WRITEDATA, NULL);
+But turning it off and leaving HEADERFUNCTION on causes a seg fault,
+I have no idea why.
+I have to leave them both on.
+Unless I write more specialized code, this brings in both data and header info,
+so I have to strip some things off after the fact.
+You'll see this after the perform function runs.
+*********************************************************************/
+
 			curl_easy_setopt(handle, CURLOPT_HEADERFUNCTION,
 					 imap_header_callback);
 			callback_data.buffer =
@@ -695,6 +780,7 @@ imap_done:
 				nzFree(mailstring);
 				goto action;
 			}
+
 /* have to strip 2 fetch BODY lines off the front,
  * and ) A018 OK off the end. */
 			t = strchr(mailstring, '\n');
@@ -710,13 +796,29 @@ imap_done:
 				t[-1] = 0, --mailstring_l;
 			t = strrchr(mailstring, '\n');
 			if (t && t - 2 >= mailstring &&
-			    !strncmp(t - 2, "\n)\nA", 4)) {
+			    !strncmp(t - 2, "\n)\n", 3) && strstr(t, " OK ")) {
 				--t;
 				*t = 0;
 				mailstring_l = t - mailstring;
 			}
+
 			key = presentMail();
 /* presentMail has already freed mailstring */
+			postkey = 0;
+		}
+
+		if (key == 'p') {
+			int j2 = j;
+			while(--j >= 0) {
+				--mif;
+				if(!mif->gone)
+					break;
+			}
+			if(j >= 0)
+				goto reaction;
+			i_puts(MSG_NoPrevMail);
+			mif = f->mlist + (j = j2);
+			goto reaction;
 		}
 
 		if (key == 'v') {
@@ -729,7 +831,7 @@ imap_done:
 
 		if (key == 'b' || key == 'f') {
 			char subkey;
-			struct FOLDER *g;
+			struct FOLDER *g = 0;
 			if (key == 'b')
 				i_printf(MSG_Batch);
 			else
@@ -780,6 +882,15 @@ imap_done:
 			goto action;
 		}
 
+		if (key == 'e') {
+			i_printf(MSG_Envelope);
+			fflush(stdout);
+			if (!fgets(inputline, sizeof(inputline), stdin))
+				goto imap_done;
+			setEnvelopeFormat(inputline);
+			goto reaction;
+		}
+
 		if (key == 'm') {
 			struct FOLDER *g;
 			i_printf(MSG_MoveTo);
@@ -811,7 +922,7 @@ imap_done:
 				} else {
 					delflag = true;
 				}
-			}
+			} else goto action;
 		}
 
 		if (key == 'd') {
@@ -842,7 +953,7 @@ imap_done:
 	}
 
 	puts("end of folder");
-}				/* scanFolder */
+}
 
 // \" is an escaped quote inside the string.
 static char *nextRealQuote(char *p)
@@ -866,11 +977,51 @@ static char *nextRealQuote(char *p)
 	return r;
 }
 
+static bool grabEmailFromEnvelope(char **t0,
+char **name, char **addr)
+{
+	char *t = *t0, *u;
+	if(!strncmp(t, "NIL ", 4)) {
+		t += 4;
+	} else {
+		if(*t != '"')
+			return false;
+		u = nextRealQuote(++t);
+		if (!u)
+			return false;
+		*u = 0;
+		if(name)
+			*name = t;
+		*t0 = t = u + 2;
+	}
+// next should come NIL "email" "domain"
+	if (strncmp(t, "NIL \"", 5))
+		return false;
+	t += 5;
+	u = strchr(t, '"');
+	if (!u)
+		return false;
+	*u = '@';
+	u += 2;
+	if (*u != '"')
+		return false;
+	strmove(strchr(t, '@') + 1, u + 1);
+	u = strchr(t, '"');
+	if (!u)
+		return false;
+	if(addr)
+		*addr = t;
+	*u = 0;
+	*t0 = u + 1;
+	return true;
+}
+
 static void envelopes(CURL * handle, struct FOLDER *f)
 {
 	int j;
 	char *t, *u;
 	CURLcode res;
+	int sublength;
 	char cust_cmd[80];
 
 	for (j = 0; j < f->nfetch; ++j) {
@@ -896,16 +1047,54 @@ static void envelopes(CURL * handle, struct FOLDER *f)
 
 		sprintf(cust_cmd, "FETCH %d ALL", mif->seqno);
 		curl_easy_setopt(handle, CURLOPT_CUSTOMREQUEST, cust_cmd);
+
+/*********************************************************************
+Originally I used the WRITEFUNCTION to ge the enveloope data.
+That returns the untagged line, and that was right 99% of the time.
+That held the entire envelope.
+But once ina while the envelope continued on the next line,
+the bodyline, the other line, whatever you call it.
+For that eventuality I have to use the HEADERFUNCTION as well.
+Both do the same thing, the same function, gather data.
+But now I get the untagged line twice.
+We'll deal with that later.
+*********************************************************************/
+
+		curl_easy_setopt(handle, CURLOPT_HEADERFUNCTION, eb_curl_callback);
+		curl_easy_setopt(handle, CURLOPT_HEADERDATA, &callback_data);
 		res = getMailData(handle);
+		curl_easy_setopt(handle, CURLOPT_HEADERFUNCTION, NULL);
+		curl_easy_setopt(handle, CURLOPT_HEADERDATA, NULL);
 		if (res != CURLE_OK) {
-//abort:
 			ebcurl_setError(res, mailbox_url, 2, emptyString);
 		}
 
+/* have to strip the untagged line off the front,
+ * as it appears twice,
+ * and A018 OK off the end. */
+		t = strchr(mailstring, '\n');
+		if (t) {
+			++t;
+			mailstring_l -= (t - mailstring);
+			strmove(mailstring, t);
+		}
+		t = mailstring + mailstring_l;
+		if (t > mailstring && t[-1] == '\n')
+			t[-1] = 0, --mailstring_l;
+		t = strrchr(mailstring, '\n');
+		if (t && strstr(t, " OK ")) {
+			++t;
+			*t = 0;
+			mailstring_l = t - mailstring;
+		}
+
 		mif->cbase = mailstring;
 		mif->subject = emptyString;
 		mif->from = emptyString;
+		mif->to = emptyString;
 		mif->reply = emptyString;
+		mif->prec = emptyString;
+		mif->ccrec = emptyString;
 
 		t = strstr(mailstring, "ENVELOPE (");
 		if (!t) {
@@ -922,20 +1111,50 @@ static void envelopes(CURL * handle, struct FOLDER *f)
 // We don't use this date because it isn't standardized,
 // it's whatever the sender's email client puts on the Date field.
 // We use INTERNALDATE later.
-		if (*t != '"')
-			continue;
-		t = strchr(++t, '"');
-		if (!t)
-			continue;
-		++t;
+		if(!strncmp(t, "NIL", 3)) {
+			t += 3;
+		} else {
+			if (*t != '"')
+				continue;
+			t = strchr(++t, '"');
+			if (!t)
+				continue;
+			++t;
+	}
 
 /* subject next, I'll assume it is always quoted */
 		while (*t == ' ')
 			++t;
-		if (*t != '"')
-			continue;
-		++t;
-		u = nextRealQuote(t);
+
+		if(!strncmp(t, "NIL ", 4)) { // missing subject
+			t += 4;
+			goto doreply;
+		}
+
+// imap sometimes has number in braces, don't know why
+		sublength = -1;
+		if(*t == '{') {
+			 sublength = strtol(t+1, &t, 10);
+			if(*t == '}')
+				++t;
+		// with number in braces, subject is on next line.
+// isspace takes us past crlf
+			while(isspace(*t))
+				++t;
+		}
+
+//  printf("%d,%d,%d|%s|\n", mailstring_l, t-mailstring, strlen(t), t);
+
+		if (*t == '"') {
+			++t;
+			u = nextRealQuote(t);
+		} else {
+			mif->line2 = true;
+			u = t + sublength;
+			if(sublength <= 0 || u - mailstring >= mailstring_l)
+				u = 0;
+		}
+
 		if (!u)
 			continue;
 		*u = 0;
@@ -944,46 +1163,46 @@ static void envelopes(CURL * handle, struct FOLDER *f)
 		mif->subject = t;
 		t = u + 1;
 
+doreply:
 		while (*t == ' ')
 			++t;
 		if (strncmp(t, "((\"", 3))
 			goto doref;
-		t += 3;
-		u = nextRealQuote(t);
-		if (!u)
+		t += 2;
+		if (!grabEmailFromEnvelope(&t, &mif->from, &mif->reply))
 			goto doref;
-		*u = 0;
-		mif->from = t;
-		t = u + 1;
 
-		while (*t == ' ')
-			++t;
-		if (strncmp(t, "NIL", 3))
+// We have parsed from-reply in block 1, block 4 contains the recipients,
+// Don't know what is in blocks 2 and 3.
+		u = strstr(t, "(("); // block 2
+		if(!u)
 			goto doref;
-		t += 3;
-		while (*t == ' ')
-			++t;
-/* again assuming each field is quoted */
-		if (*t != '"')
+		t = u + 2;
+		u = strstr(t, "(("); // block 3
+		if(!u)
 			goto doref;
-		++t;
-		u = strchr(t, '"');
-		if (!u)
+		t = u + 2;
+		u = strstr(t, "(("); // block 4
+		if(!u)
 			goto doref;
-		*u = '@';
-		++u;
-		while (*u == ' ')
-			++u;
-		if (*u != '"')
+		t = u + 2;
+		if (!grabEmailFromEnvelope(&t, &mif->to, &mif->prec))
 			goto doref;
-		++u;
-		strmove(strchr(t, '@') + 1, u);
-		u = strchr(t, '"');
-		if (!u)
+
+// block 5 is the carbon copies, I guess, I don't know.
+// It doesn't have to be there.
+// Block 6 is bcc.
+#if 0
+		u = strstr(t, ")) (("); // block 5
+		u = strstr(t, ")) NIL (("); // block 6
+		if(!u)
 			goto doref;
-		*u = 0;
-		mif->reply = t;
-		t = u + 1;
+		t = u + 5;
+		if (!grabEmailFromEnvelope(&t, &f->ccrec, 0))
+			goto doref;
+#endif
+
+//  printf("%s %s %s %s\n", mif->from, mif->reply, mif->to, mif->prec);
 
 doref:
 /* find the reference string, for replies */
@@ -1042,7 +1261,7 @@ dosize:
 		mif->size = atoi(t);
 
 	}
-}				/* envelopes */
+}
 
 /* examine the specified folder, gather message envelopes */
 static void examineFolder(CURL * handle, struct FOLDER *f, bool dostats)
@@ -1119,7 +1338,7 @@ static void examineFolder(CURL * handle, struct FOLDER *f, bool dostats)
 		i_printf(MSG_ShowLast + earliest, f->nfetch, f->nmsgs);
 	else
 		i_printf(MSG_MessagesX, f->nmsgs);
-}				/* examineFolder */
+}
 
 /* find the last mail in the local unread directory */
 static int unreadMax, unreadMin, unreadCount;
@@ -1146,7 +1365,7 @@ static void unreadStats(void)
 			++unreadCount;
 		}
 	}
-}				/* unreadStats */
+}
 
 static char *umf;		/* unread mail file */
 static char *umf_end;
@@ -1179,7 +1398,7 @@ static CURL *newFetchmailHandle(const char *username, const char *password)
 		ebcurl_setError(res, mailbox_url, 2, emptyString);
 
 	return handle;
-}				/* newFetchmailHandle */
+}
 
 static void get_mailbox_url(const struct MACCOUNT *account)
 {
@@ -1189,8 +1408,10 @@ static void get_mailbox_url(const struct MACCOUNT *account)
 	if (account->inssl)
 		scheme = "pop3s";
 
-	if (isimap)
+	if (isimap) {
 		scheme = (account->inssl ? "imaps" : "imap");
+		loadAddressBook();
+	}
 
 	if (asprintf(&url,
 		     "%s://%s:%d/", scheme, account->inurl,
@@ -1201,7 +1422,7 @@ static void get_mailbox_url(const struct MACCOUNT *account)
 			     strlen(scheme) + strlen(account->inurl) + 8);
 	}
 	mailbox_url = url;
-}				/* get_mailbox_url */
+}
 
 /* reads message into mailstring, it's up to you to free it */
 static CURLcode fetchOneMessage(CURL * handle, int message_number)
@@ -1233,7 +1454,7 @@ static CURLcode fetchOneMessage(CURL * handle, int message_number)
 	close(umfd);
 
 	return res;
-}				/* fetchOneMessage */
+}
 
 static CURLcode deleteOneMessage(CURL * handle)
 {
@@ -1245,7 +1466,7 @@ static CURLcode deleteOneMessage(CURL * handle)
 		return res;
 	res = getMailData(handle);
 	return res;
-}				/* deleteOneMessage */
+}
 
 static CURLcode count_messages(CURL * handle, int *message_count)
 {
@@ -1289,7 +1510,8 @@ refresh:
 			goto input;
 		}
 
-		if (stringEqual(inputline, "q")) {
+		if (stringEqual(inputline, "q")
+		|| stringEqual(inputline, "qt")) {
 			i_puts(MSG_Quit);
 			goto imap_done;
 		}
@@ -1307,6 +1529,15 @@ refresh:
 			goto input;
 		}
 
+		if (*t == 'e') {
+			if(!t[1]) {
+				setEnvelopeFormat("");
+				goto input;
+			}
+			if(isspace(t[1]) && setEnvelopeFormat(t + 2))
+				goto input;
+		}
+
 		if (!strncmp(t, "create ", 7)) {
 			char *w;
 			t += 7;
@@ -1394,7 +1625,7 @@ refresh:
 
 	*message_count = num_messages;
 	return CURLE_OK;
-}				/* count_messages */
+}
 
 /* Returns number of messages fetched */
 int fetchMail(int account)
@@ -1408,6 +1639,9 @@ int fetchMail(int account)
 	const char *url_for_error;
 	int message_count = 0, message_number;
 
+// remember the envelope format we got from the config file
+	strcpy(envelopeFormatDef, envelopeFormat);
+
 	get_mailbox_url(a);
 	url_for_error = mailbox_url;
 
@@ -1459,7 +1693,7 @@ fetchmail_cleanup:
 	nzFree(mailstring);
 	mailstring = initString(&mailstring_l);
 	return nfetch;
-}				/* fetchMail */
+}
 
 /* fetch from all accounts except those with nofetch or imap set */
 int fetchAllMail(void)
@@ -1489,7 +1723,7 @@ int fetchAllMail(void)
 	}
 
 	return nfetch;
-}				/* fetchAllMail */
+}
 
 static void readReplyInfo(void);
 static void writeReplyInfo(const char *addstring);
@@ -1538,7 +1772,7 @@ void scanMail(void)
 	}			/* loop over mail messages */
 
 	exit(0);
-}				/* scanMail */
+}
 
 /* a mail message is in mailstring, present it to the user */
 /* Return the key that was pressed.
@@ -1607,7 +1841,7 @@ static char presentMail(void)
 /* display the next page of mail and get a command from the keyboard */
 	displine = 1;
 paging:
-	if (!delflag) {		/* show next page */
+	if (!delflag && !postkey) {		/* show next page */
 		if (displine <= cw->dol) {
 			for (j = 0; j < 20 && displine <= cw->dol;
 			     ++j, ++displine) {
@@ -1625,12 +1859,16 @@ key_command:
 	if (delflag)
 		goto writeMail;
 
+	if(postkey) {
+		key = postkey, postkey = 0;
+	} else {
 /* interactive prompt depends on whether there is more text or not */
-	printf("%c ", displine > cw->dol ? '?' : '*');
-	fflush(stdout);
-	key = getLetter((isimap ? "qvbfh? nwWuUasdm" : "qh? nwud"));
-	printf("\b\b\b");
-	fflush(stdout);
+		printf("%c ", displine > cw->dol ? '?' : '*');
+		fflush(stdout);
+		key = getLetter((isimap ? "qvbfh? npwWuUasdm" : "qh? nwud"));
+		printf("\b\b\b");
+		fflush(stdout);
+	}
 
 	switch (key) {
 	case 'q':
@@ -1641,6 +1879,10 @@ key_command:
 		i_puts(MSG_Next);
 		goto afterinput;
 
+	case 'p':
+		i_puts(MSG_Previous);
+		goto afterinput;
+
 	case 's':
 	case 'm':
 	case 'v':
@@ -1810,8 +2052,8 @@ afterinput:
 
 	if (delflag)
 		return 'd';
-	return strchr("smvbf", key) ? key : 'n';
-}				/* presentMail */
+	return strchr("smvbfp", key) ? key : 'n';
+}
 
 /* Here are the common keywords for mail header lines.
  * These are in alphabetical order, so you can stick more in as you find them.
@@ -1907,7 +2149,7 @@ bool emailTest(void)
 	}			/* loop over lines */
 
 	return false;
-}				/* emailTest */
+}
 
 void mail64Error(int err)
 {
@@ -1919,7 +2161,7 @@ void mail64Error(int err)
 		runningError(MSG_AttAfterChars);
 		break;
 	}			/* switch on error code */
-}				/* mail64Error */
+}
 
 static void unpackQP(struct MHINFO *w)
 {
@@ -1947,7 +2189,7 @@ static void unpackQP(struct MHINFO *w)
 	}
 	w->end = r;
 	*r = 0;
-}				/* unpackQP */
+}
 
 /* Look for the name of the attachment and boundary */
 static void ctExtras(struct MHINFO *w, const char *s, const char *t)
@@ -1982,6 +2224,13 @@ static void ctExtras(struct MHINFO *w, const char *s, const char *t)
 			if (ar - al >= MHLINE)
 				ar = al + MHLINE - 1;
 			strncpy(w->cfn, al, ar - al);
+// for security reasons, don't allow an absolute path, or even relative path
+// away from the current directory or the download directory.
+// tr / _    this seems to be what other clients do.
+			char *e;
+			for(e = w->cfn; *e; ++e)
+				if(*e == '/')
+					*e = '_';
 			break;
 		}
 	}
@@ -2012,7 +2261,7 @@ static void ctExtras(struct MHINFO *w, const char *s, const char *t)
 			break;
 		}
 	}			/* multi or alt */
-}				/* ctExtras */
+}
 
 static void isoDecode(char *vl, char **vrp)
 {
@@ -2110,7 +2359,7 @@ finish:
 	}
 
 	*vrp = vr;
-}				/* isoDecode */
+}
 
 /* mail header reformat, to/from utf8 */
 static void mhReformat(char *line)
@@ -2124,7 +2373,7 @@ static void mhReformat(char *line)
 		tbuf[MHLINE - 1] = 0;
 	strcpy(line, tbuf);
 	nzFree(tbuf);
-}				/* mhReformat */
+}
 
 static void extractLessGreater(char *s)
 {
@@ -2135,7 +2384,7 @@ static void extractLessGreater(char *s)
 		*vr = 0;
 		strmove(s, vl + 1);
 	}
-}				/* extractLessGreater */
+}
 
 /* Now that we know it's mail, see what information we can
  * glean from the headers.
@@ -2147,7 +2396,6 @@ static struct MHINFO *headerGlean(char *start, char *end)
 	char *vl, *vr;		/* value left and value right */
 	struct MHINFO *w;
 	int j, k, n;
-	char linetype = 0;
 
 /* defaults */
 	w = allocZeroMem(sizeof(struct MHINFO));
@@ -2159,72 +2407,52 @@ static struct MHINFO *headerGlean(char *start, char *end)
 	w->cclist = initString(&w->cclen);
 	w->start = start, w->end = end;
 
+// join lines together
+	for(s = t = start; s < end; ++s) {
+		if(*s == '\n') {
+			if(s == end - 1 || s[1] == '\n')
+				break;
+			if(s[1] == ' ' || s[1] == '\t')
+				continue; // join lines together
+			}
+		*t++ = *s;
+	}
+
+// I don't know what to do with the gap.
+	while(t < s)
+		*t++ = '\n';
+
+// step through the header lines
 	for (s = start; s < end; s = t + 1) {
 		char quote;
-		char first = *s;
 		t = strchr(s, '\n');
 		if (!t)
 			t = end - 1;	/* should never happen */
 		if (t == s)
-			break;	/* empty line */
+			break;	// empty line, end of headers
 
-		if (first == ' ' || first == '\t') {
-			if (linetype == 'c')
-				ctExtras(w, s, t);
-			if (linetype == 't')
-				stringAndBytes(&w->tolist, &w->tolen, s, t - s);
-			if (linetype == 'y')
-				stringAndBytes(&w->cclist, &w->cclen, s, t - s);
-			if (linetype == 's') {
-				int l1, l2;
-				++s;
-				l1 = strlen(w->subject);
-				l2 = t - s;
-				if(l1 + l2 > MHLINE)
-					l2 = MHLINE - l1;
-				strncpy(w->subject + l1, s, l2);
-				if (t == end - 1 || (t[1] != ' ' && t[1] != '\t')) {
-					vl = w->subject;
-					mhReformat(vl);
-					vr = vl + strlen(vl);
-					isoDecode(vl, &vr);
-					*vr = 0;
-				}
-			}
-			continue;
-		}
-
-/* find the lead word */
+// find the lead word
 		for (q = s; isalnumByte(*q) || *q == '_' || *q == '-'; ++q) ;
 		if (q == s)
-			continue;	/* should never happen */
+			continue;	// should never happen
 		if (*q++ != ':')
-			continue;	/* should never happen */
+			continue;	// should never happen
 		for (vl = q; *vl == ' ' || *vl == '\t'; ++vl) ;
-		if (vl == t && t < end - 1 && (t[1] == ' ' || t[1] == '\t')) {
-// foobar: and that's all, maybe on the next line?
-			t = strchr(t + 1, '\n');
-			if (!t)
-				t = end - 1;	/* should never happen */
-			for (++vl; *vl == ' ' || *vl == '\t'; ++vl) ;
-		}
 		for (vr = t; vr > vl && (vr[-1] == ' ' || vr[-1] == '\t');
 		     --vr) ;
 		if (vr == vl)
-			continue;	/* empty */
+			continue;	// empty
 
-/* too long? */
+// is it too long?
+// Should print out an error or something.
 		if (vr - vl > MHLINE - 1)
 			vr = vl + MHLINE - 1;
 
 /* This is sort of a switch statement on the word */
 		if (memEqualCI(s, "subject:", q - s)) {
-			linetype = 's';
-			if (w->subject[0]) {
-				linetype = 0;
+			if (w->subject[0])
 				continue;
-			}
-/* get rid of forward/reply prefixes */
+// get rid of forward/reply prefixes
 			for (q = vl; q < vr; ++q) {
 				static const char *const prefix[] = {
 					"re", "sv", "fwd", 0
@@ -2250,39 +2478,33 @@ static struct MHINFO *headerGlean(char *start, char *end)
 				--q;	/* try again */
 			}
 			strncpy(w->subject, vl, vr - vl);
-			if (t == end - 1 || (t[1] != ' ' && t[1] != '\t')) {
-				vl = w->subject;
-				mhReformat(vl);
-				vr = vl + strlen(vl);
-				isoDecode(vl, &vr);
-				*vr = 0;
-			}
+			vl = w->subject;
+			mhReformat(vl);
+			vr = vl + strlen(vl);
+			isoDecode(vl, &vr);
+			*vr = 0;
 			continue;
 		}
 
 		if (memEqualCI(s, "reply-to:", q - s)) {
-			linetype = 'r';
 			if (!w->reply[0])
 				strncpy(w->reply, vl, vr - vl);
 			continue;
 		}
 
 		if (memEqualCI(s, "message-id:", q - s)) {
-			linetype = 'm';
 			if (!w->mid[0])
 				strncpy(w->mid, vl, vr - vl);
 			continue;
 		}
 
 		if (memEqualCI(s, "references:", q - s)) {
-			linetype = 'e';
 			if (!w->ref[0])
 				strncpy(w->ref, vl, vr - vl);
 			continue;
 		}
 
 		if (memEqualCI(s, "from:", q - s)) {
-			linetype = 'f';
 			if (w->from[0])
 				continue;
 			isoDecode(vl, &vr);
@@ -2293,10 +2515,9 @@ static struct MHINFO *headerGlean(char *start, char *end)
 
 		if (memEqualCI(s, "date:", q - s)
 		    || memEqualCI(s, "sent:", q - s)) {
-			linetype = 'd';
 			if (w->date[0])
 				continue;
-/* don't need the weekday, seconds, or timezone */
+// don't need the weekday, seconds, or timezone
 			if (vr - vl > 5 &&
 			    isalphaByte(vl[0]) && isalphaByte(vl[1])
 			    && isalphaByte(vl[2]) && vl[3] == ','
@@ -2310,14 +2531,13 @@ static struct MHINFO *headerGlean(char *start, char *end)
 		}
 
 		if (memEqualCI(s, "to:", q - s)) {
-			linetype = 't';
 			if (w->tolen)
 				stringAndChar(&w->tolist, &w->tolen, ',');
 			stringAndBytes(&w->tolist, &w->tolen, q, vr - q);
 			if (w->to[0])
 				continue;
 			strncpy(w->to, vl, vr - vl);
-/* Only retain the first recipient */
+// Only retain the first recipient
 			quote = 0;
 			for (q = w->to; *q; ++q) {
 				if (*q == ',' && !quote) {
@@ -2347,7 +2567,6 @@ static struct MHINFO *headerGlean(char *start, char *end)
 		}
 
 		if (memEqualCI(s, "cc:", q - s)) {
-			linetype = 'y';
 			if (w->cclen)
 				stringAndChar(&w->cclist, &w->cclen, ',');
 			stringAndBytes(&w->cclist, &w->cclen, q, vr - q);
@@ -2356,7 +2575,6 @@ static struct MHINFO *headerGlean(char *start, char *end)
 		}
 
 		if (memEqualCI(s, "content-type:", q - s)) {
-			linetype = 'c';
 			if (memEqualCI(vl, "application/pgp-signature", 25))
 				w->pgp = true;
 			if (memEqualCI(vl, "text", 4))
@@ -2377,7 +2595,6 @@ static struct MHINFO *headerGlean(char *start, char *end)
 		}
 
 		if (memEqualCI(s, "content-transfer-encoding:", q - s)) {
-			linetype = 'e';
 			if (memEqualCI(vl, "quoted-printable", 16))
 				w->ce = CE_QP;
 			if (memEqualCI(vl, "7bit", 4))
@@ -2389,8 +2606,7 @@ static struct MHINFO *headerGlean(char *start, char *end)
 			continue;
 		}
 
-		linetype = 0;
-	}			/* loop over lines */
+	}			// loop over lines
 
 /* make sure there's room for a final nl */
 	stringAndChar(&w->tolist, &w->tolen, ' ');
@@ -2595,7 +2811,7 @@ textonly:
 	w->start = start = s;
 
 	return w;
-}				/* headerGlean */
+}
 
 static char *headerShow(struct MHINFO *w, bool top)
 {
@@ -2667,7 +2883,7 @@ static char *headerShow(struct MHINFO *w, bool top)
 				strcat(buf, "<br>");
 			lines = true;
 			if (nimages) {
-				sprintf(atbuf, "%d images\n", nimages);
+				sprintf(atbuf, "%d images", nimages);
 				if (nimages == 1)
 					strcpy(atbuf, "1 image");
 				strcat(buf, atbuf);
@@ -2682,13 +2898,12 @@ static char *headerShow(struct MHINFO *w, bool top)
 				}
 			}
 			if (nattach > nimages + 1) {
-				sprintf(atbuf, "%d attachments\n",
+				sprintf(atbuf, "%d attachments",
 					nattach - nimages);
 				strcat(buf, atbuf);
 			}
 			strcat(buf, "\n");
 		}
-		/* attachments */
 		if (w->to[0] && !ismc) {
 			if (lines & mailIsHtml)
 				strcat(buf, "<br>");
@@ -2729,7 +2944,7 @@ static char *headerShow(struct MHINFO *w, bool top)
 		strcat(buf, mailIsHtml ? "<P>\n" : "\n");
 	strcpy(lastsubject, w->subject);
 	return buf;
-}				/* headerShow */
+}
 
 /* Depth first block of text determines the type */
 static int mailTextType(struct MHINFO *w)
@@ -2927,7 +3142,7 @@ bool setupReply(bool all)
 
 		p = (char *)fetchLine(j, 1);
 
-		if (memEqualCI(p, "subject:", 8)) {
+		if (memEqualCI(p, "subject:", 8) || memEqualCI(p, "sub:", 4)) {
 			linetype[j] = 's';
 			subln = j;
 			goto nextline;
