@@ -39,7 +39,7 @@ void prepareForBrowse(char *h, int h_len)
 		h[j++] = h[i];
 	}
 	h[j] = 0;
-}				/* prepareForBrowse */
+}
 
 /* An input field cannot contain newline, null, or the InternalCodeChar */
 // Revised June 2018, maybe newline is ok. We need it for textarea.
@@ -52,7 +52,7 @@ void prepareForField(char *h)
 			*h = InternalCodeCharAlternate;
 		++h;
 	}
-}				/* prepareForField */
+}
 
 /*********************************************************************
 The primary goal of this routine is to turn
@@ -88,6 +88,10 @@ static void cellDelimiters(char *buf)
 	char *s;
 
 	for (s = buf; *s; ++s) {
+		if (*s == DataCellChar) {
+			*s = '|';
+			continue;
+		}
 		if (*s == TableCellChar) {
 			*s = '|';
 			lastcell = s;
@@ -114,11 +118,13 @@ static void anchorSwap(char *buf)
 	bool strong;		// strong whitespace, newline or paragraph
 	int n, cnt;
 	char tag[20];
+	const char *pua; // for private use area
 
-	static const char from[] =
-	    "\x1b\x95\x99\x9c\x9d\x91\x92\x93\x94\xa0\xad\x96\x97\x85";
-	static const char becomes[] = "_*'`'`'`' ----";
-/* I use to convert a6 and c2 to hyphen space, not sure why */
+// this is transliteration, like /bin/tr
+// I use to convert a6 and c2 to hyphen space, not sure why
+	static const char from[] = "\x1b\x95\x99\x9c\x9d\x91\x92\x93\x94\xa0\xad\x96\x97\x85";
+// x goes away, like tr -d
+	static const char becomes[] = "_*'`'`'`' x---";
 
 /* Transliterate a few characters.  One of them is 0xa0 to space,
  * so we need to do this now, before the anchors swap with whitespace.
@@ -128,11 +134,14 @@ static void anchorSwap(char *buf)
  * Don't do any of these transliterations in an input field. */
 
 	inputmode = false;
+	pua = getenv("EB_PUA");
+	if(pua && !*pua) pua = 0;
+
 	for (s = w = buf; (c = *s); ++s) {
 		d = s[1];
 		if (c == InternalCodeChar && isdigitByte(d)) {
-			strtol(s + 1, &ss, 10);
-			if (*ss == '<')
+			int tagno = strtol(s + 1, &ss, 10);
+			if (*ss == '<' && !stringEqual(tagList[tagno]->info->name, "button"))
 				inputmode = true;
 			if (*ss == '>')
 				inputmode = false;
@@ -149,7 +158,7 @@ static void anchorSwap(char *buf)
 
 /* utf8 test */
 		if ((c & 0xc0) == 0xc0 && (d & 0xc0) == 0x80) {
-			unsigned int uni = 0;
+			unsigned int uni = 0, ubytes = 0;
 			if ((c & 0x3c) == 0) {
 /* fits in 8 bits */
 				uni = ((uchar) c << 6) | (d & 0x3f);
@@ -157,25 +166,45 @@ static void anchorSwap(char *buf)
 				if (ss) {
 					c = becomes[ss - from];
 					++s;
+					if(c == 'x')
+						continue;
 					goto put1;
 				}
 			}
 /* copy the utf8 sequence as is */
+			uni = 0;
 			*w++ = c;
 			++s;
-			c <<= 1;
+			c = (char) ((uchar)c << 1);
 			while ((c & 0x80) && ((d = *s) & 0xc0) == 0x80) {
-				*w++ = d;
-				++s;
+				*w++ = d, ++s, ++ubytes;
+				uni = (uni << 6) | (d & 0x3f);
+				c = (char) ((uchar)c << 1);
 			}
 			--s;
+				c = (char) ((uchar)c >> (1+ubytes));
+				uni |= ((unsigned int)c << (ubytes*6));
+// We can do things with high unicodes here, if we wish.
+
+// Suppresse private unicodes, which shouldn't appear on public websites,
+// and if they do, there isn't a consistent way to read them.
+// set EB_PUA=on if you want to retain them.
+			if(((uni >= 0xe000 && uni < 0xf8ff) ||
+			(uni >= 0xf0000 && uni < 0xffffd) ||
+			(uni >= 0x100000 && uni < 0x10fffd)) &&
+			!pua)
+				w -= ubytes+1;
+
 			continue;
 		}
 
 /* Now assuming iso8859-1, which is practically deprecated */
 		ss = strchr(from, c);
-		if (ss)
+		if (ss) {
 			c = becomes[ss - from];
+			if(c == 'x')
+				continue;
+		}
 
 #if 0
 // Should we modify empty anchors in any way?
@@ -412,7 +441,7 @@ hence it would send a newline if there was one.
 		}
 		s = a;
 	}
-}				/* anchorSwap */
+}
 
 /*********************************************************************
 Format text, and break lines at sentence/phrase boundaries.
@@ -470,7 +499,7 @@ static void debugChunk(const char *chunk, int len)
 		fprintf(f, "%c", c);
 	}
 	fprintf(f, ">%d.%d\n", colno, lspace);
-}				/* debugChunk */
+}
 
 static void appendOneChar(char c)
 {
@@ -478,7 +507,7 @@ static void appendOneChar(char c)
 		bl_overflow = true;
 	else
 		*bl_cursor++ = c;
-}				/* appendOneChar */
+}
 
 static bool spaceNotInInput(void)
 {
@@ -501,7 +530,7 @@ static bool spaceNotInInput(void)
 			return false;
 	}
 	return true;
-}				/* spaceNotInInput */
+}
 
 static void appendSpaceChunk(const char *chunk, int len, bool premode)
 {
@@ -626,7 +655,7 @@ static void appendSpaceChunk(const char *chunk, int len, bool premode)
 			colno += 4;
 	}
 	lspace = 1;
-}				/* appendSpaceChunk */
+}
 
 static void appendPrintableChunk(const char *chunk, int len, bool premode)
 {
@@ -643,7 +672,7 @@ static void appendPrintableChunk(const char *chunk, int len, bool premode)
 		if (visible) {
 // each foreign char or emoji counts as one.
 // Ignore all but the first byte of a utf8.
-			if ((char)c >= 0	// ascii
+			if (!(c&0x80)	// ascii
 			    || (c & 0x40) == 0x40)
 				++colno;
 			continue;
@@ -682,7 +711,7 @@ static void appendPrintableChunk(const char *chunk, int len, bool premode)
 	lcomma -= i;
 	lright -= i;
 	lany -= i;
-}				/* appendPrintableChunk */
+}
 
 /* Break up a line using the above routines.
  * The new lines are put in a fixed array.
@@ -1037,7 +1066,7 @@ bool isEmailAddress(const char *s)
 		return false;
 	for (; *s; ++s) {
 		char c = *s;
-		if (c < 0)	// nonascii
+		if (c&0x80)	// nonascii
 			return false;
 		if (atfound) {
 			if (!isalnum(c) && c != '.' && c != '-')
@@ -1124,7 +1153,7 @@ binchar:
 	}
 
 	return (bincount * 8 - 16 >= charcount);
-}				/* looksBinary */
+}
 
 void looks_8859_utf8(const uchar * buf, int buflen, bool * iso_p, bool * utf8_p)
 {
@@ -1146,7 +1175,7 @@ isogo:
 		if ((buf[i + 1] & 0xc0) != 0x80)
 			goto isogo;
 		c <<= 2;
-		for (j = i + 2; c < 0; ++j, c <<= 1)
+		for (j = i + 2; c&0x80; ++j, c <<= 1)
 			if ((buf[j] & 0xc0) != 0x80)
 				goto isogo;
 		++utfcount;
@@ -1161,9 +1190,11 @@ isogo:
 	bothcount *= 6;
 	if (utfcount * 7 >= bothcount)
 		*utf8_p = true;
-	if (isocount * 7 >= bothcount)
+// a sequence of ascii, like times +- 7, can look like utf8.
+// If 2/3 of the nonascii chars look like iso, I'll call it iso.
+	if (isocount * 9 >= bothcount)
 		*iso_p = true;
-}				/* looks_8859_utf8 */
+}
 
 /*********************************************************************
 Convert a string from iso 8859 to utf8, or vice versa.
@@ -1172,7 +1203,7 @@ Don't forget to free it when you're done.
 *********************************************************************/
 
 /* only 8859-1 and 8859-2 so far */
-static const int iso_unicodes[2][128] = {
+static const unsigned int iso_unicodes[2][128] = {
 	{
 /*********************************************************************
 The first 32 nonascii chars in iso8859-1 are control characters,
@@ -1229,7 +1260,7 @@ void iso2utf(const uchar * inbuf, int inbuflen, uchar ** outbuf_p,
 	int nacount = 0;
 	uchar c;
 	uchar *outbuf;
-	const int *isoarray = iso_unicodes[type8859 - 1];
+	const unsigned int *isoarray = iso_unicodes[type8859 - 1];
 	int ucode;
 	char *s;
 
@@ -1266,7 +1297,7 @@ void iso2utf(const uchar * inbuf, int inbuflen, uchar ** outbuf_p,
 
 	*outbuf_p = outbuf;
 	*outbuflen_p = j;
-}				/* iso2utf */
+}
 
 void utf2iso(const uchar * inbuf, int inbuflen, uchar ** outbuf_p,
 	     int *outbuflen_p)
@@ -1274,8 +1305,8 @@ void utf2iso(const uchar * inbuf, int inbuflen, uchar ** outbuf_p,
 	int i, j, k;
 	uchar c;
 	uchar *outbuf;
-	const int *isoarray = iso_unicodes[type8859 - 1];
-	int ucode;
+	const unsigned int *isoarray = iso_unicodes[type8859 - 1];
+	unsigned int ucode;
 
 	if (!inbuflen) {
 		*outbuf_p = (uchar *) emptyString;
@@ -1331,7 +1362,7 @@ void utf2iso(const uchar * inbuf, int inbuflen, uchar ** outbuf_p,
 /* unicodes not found in our iso class are converted into stars */
 		c <<= 1;
 		++i;
-		for (++i; c < 0; ++i, c <<= 1) {
+		for (++i; c&0x80; ++i, c <<= 1) {
 			if ((outbuf[i] & 0xc0) != 0x80)
 				break;
 		}
@@ -1342,7 +1373,52 @@ void utf2iso(const uchar * inbuf, int inbuflen, uchar ** outbuf_p,
 
 	*outbuf_p = outbuf;
 	*outbuflen_p = j;
-}				/* utf2iso */
+}
+
+// like the above, but always iso8859-1, and the change is made inline
+void utf2iso1(char *s0, size_t *lenp)
+{
+	char *s = s0;
+	char *t = s0;
+// null pointer means we don't specify a length
+	while((!lenp && *s) || (lenp && (unsigned)(s - s0) < *lenp)) {
+		uchar c = *s;
+		uchar d = s[1];
+		if((c&0xe0) == 0xe0 || (c&0xe0) == 0x80 ||
+		((c&0x80) && (d&0xc0) != 0x80)) {
+			debugPrint(1, "improper utf8 format in payload");
+			break;
+		}
+		if(c >= 0xc4) {
+			debugPrint(1, "null or high character in payload");
+			break;
+		}
+		if(c < 0x80) { *t++ = c; ++s; continue; }
+		*t++ = ((d&0x3f) | (c<<6));
+		s += 2;
+	}
+	*t = 0;
+	if(lenp) *lenp = t - s0;
+}
+
+// reverse the above. This time the new string is allocated, as it might be longer
+char *iso12utf(const char *t1, const char *t2, int *lenp)
+{
+	int l;
+	char *s = initString(&l);
+	while(t1 < t2) {
+		uchar c = (uchar)*t1;
+		if(!(c&0x80)) { // ascii
+			stringAndChar(&s, &l, c);
+		} else {
+			stringAndChar(&s, &l, (0xc2 | ((c>>6)&1)));
+			stringAndChar(&s, &l, (0x80|(c&0x3f)));
+		}
+		++t1;
+	}
+	*lenp = l;
+	return s;
+}
 
 /*********************************************************************
 Convert the current line in buffer, which is either iso8859-1 or utf8,
@@ -1442,7 +1518,7 @@ void utfHigh(const char *inbuf, int inbuflen, char **outbuf_p, int *outbuflen_p,
 
 	*outbuf_p = (char *)outbuf;
 	*outbuflen_p = j;
-}				/* utfHigh */
+}
 
 /* convert a 32 bit unicode character into utf8 */
 char *uni2utf8(unsigned int unichar)
@@ -1481,7 +1557,7 @@ char *uni2utf8(unsigned int unichar)
 
 	outbuf[n] = 0;
 	return (char *)outbuf;
-}				/* uni2utf8 */
+}
 
 void utfLow(const char *inbuf, int inbuflen, char **outbuf_p, int *outbuflen_p,
 	    int bom)
@@ -1491,7 +1567,7 @@ void utfLow(const char *inbuf, int inbuflen, char **outbuf_p, int *outbuflen_p,
 	unsigned int unicode;
 	int isbig;
 	int k, l;
-	const int *isoarray = iso_unicodes[type8859 - 1];
+	const unsigned int *isoarray = iso_unicodes[type8859 - 1];
 
 	if (!inbuflen) {
 		*outbuf_p = emptyString;
@@ -1594,7 +1670,7 @@ void utfLow(const char *inbuf, int inbuflen, char **outbuf_p, int *outbuflen_p,
 
 	*outbuf_p = obuf;
 	*outbuflen_p = obuf_l - 2;
-}				/* utfLow */
+}
 
 // Convert from whatever it is to utf8, for javascript and css.
 // Result parameter is the new string, or null if no conversion.
@@ -1649,7 +1725,7 @@ char *base64Encode(const char *inbuf, int inlen, bool lines)
 	int outlen = ((inlen / 3) + 1) * 4;
 	++outlen;		/* zero on the end */
 	if (lines)
-		outlen += (inlen / 54) + 1;
+		outlen += ((inlen / 54) + 1) * 2;
 	outstr = out = allocMem(outlen);
 	colno = 0;
 	while (inlen >= 3) {
@@ -1665,7 +1741,7 @@ char *base64Encode(const char *inbuf, int inlen, bool lines)
 		colno += 4;
 		if (colno < 72)
 			continue;
-		*out++ = '\n';
+		*out++ = '\r', *out++ = '\n';
 		colno = 0;
 	}
 	if (inlen == 1) {
@@ -1684,10 +1760,10 @@ char *base64Encode(const char *inbuf, int inlen, bool lines)
 	}
 /* finish the last line */
 	if (lines && colno)
-		*out++ = '\n';
+		*out++ = '\r', *out++ = '\n';
 	*out = 0;
 	return outstr;
-}				/* base64Encode */
+}
 
 uchar base64Bits(char c)
 {
@@ -1702,7 +1778,7 @@ uchar base64Bits(char c)
 	if (c == '/')
 		return 63;
 	return 64;		/* error */
-}				/* base64Bits */
+}
 
 /*********************************************************************
 Decode some data in base64.
@@ -1763,7 +1839,7 @@ int base64Decode(char *start, char **end)
 	}
 	*end = r;
 	return ret;
-}				/* base64Decode */
+}
 
 void
 iuReformat(const char *inbuf, int inbuflen, char **outbuf_p, int *outbuflen_p)
@@ -1786,7 +1862,30 @@ iuReformat(const char *inbuf, int inbuflen, char **outbuf_p, int *outbuflen_p)
 		utf2iso((uchar *) inbuf, inbuflen, (uchar **) outbuf_p,
 			outbuflen_p);
 	}
-}				/* iuReformat */
+}
+
+// Lines have to be doslike when sending multipart/form-data.
+// Question: is this also true when sending text attachments by email?
+// Free the input string and return the output string allocated.
+char *makeDosNewlines(char *p)
+{
+	int linecount = 0;
+	char *s, *t, *p2;
+	for(s = p; *s; ++s)
+		if(*s == '\n' && s > p && s[-1] != '\r')
+			++linecount;
+	if(!linecount)
+		return p;
+	t = p2 = allocMem(strlen(p) + linecount + 1);
+	for(s = p; *s; ++s) {
+		if(*s == '\n' && s > p && s[-1] != '\r')
+			*t++ = '\r';
+		*t++ = *s;
+	}
+*t = 0;
+	nzFree(p);
+	return p2;
+}
 
 bool parseDataURI(const char *uri, char **mediatype, char **data, int *data_l)
 {
@@ -2077,3 +2176,395 @@ char *closeColor(const char *s)
 fail:
 	return 0;
 }
+
+/*********************************************************************
+Read in emojis from a library.
+These are hierarchical - a group, then the emojis in that group.
+heart.green  heart.blue  etc.  Like Carl Linnaeus.
+No more than 100 emojis per group.
+This seems arbitrary - but if you just wanna review the emojis in a group,
+cause you don't remember them, you don't want a menu with hundreds of choices.
+Maybe 100 could be larger, but for practical considerations,
+there should be a limit.
+*********************************************************************/
+
+#define EJGROUPSIZE 100
+
+static struct EJGROUP {
+	struct EJGROUP *next;
+	int n;
+	const char *name;
+	const char *desc[EJGROUPSIZE];
+	long code[EJGROUPSIZE];
+} *ejgroup0;
+// the emojis file pulled into memory
+static char *ejbase;
+
+void clearEmojis(void)
+{
+	struct EJGROUP *g, *g1;
+	nzFree(ejbase);
+	ejbase = 0;
+	g = ejgroup0;
+	while(g) {
+		g1 = g->next;
+		free(g);
+		g = g1;
+	}
+ejgroup0 = 0;
+}
+
+void loadEmojis(void)
+{
+	int i, j, len, lineno;
+	char c;
+	uchar state;
+	struct EJGROUP *g = 0;
+	char *s, *t, *u;
+
+	if(!emojiFile) // should never happen
+		return;
+	if(ejgroup0) // should never happen
+		clearEmojis();
+	if(!fileIntoMemory(emojiFile, &ejbase, &len)) {
+		showError();
+		setError(-1);
+		return;
+	}
+
+// look for bad characters, and compress whitespace
+	state = 2, lineno = 1;
+	for(i = j = 0; i < len; ++i) {
+		c = ejbase[i];
+		if(state == 3 && c != '\n') continue;
+		if(c == '\t' || c == '\r' || c == '\f')
+			c = ' ';
+		if(c == ' ') {
+			if(!state) ejbase[j++] = c, state = 1;
+			continue;
+		}
+		if(c == '\n') {
+			if(state == 1) --j;
+			ejbase[j++] = c;
+			state = 2, ++lineno;
+			continue;
+		}
+		if(c == '#' && state == 2) {
+			state = 3; continue;
+		}
+// all the whitespace has been dealt with
+// I use to disallow nonascii, but other languages might
+// employ accented letters, as utf8.
+#if 0
+		if((signed char)c <= ' ') {
+			i_printf(MSG_EmojiNonascii, lineno);
+			goto fail;
+		}
+#endif
+		if((uchar) c < ' ') {
+			i_printf(MSG_EmojiBadControl, c, lineno);
+			goto fail;
+		}
+		if((signed char)c > ' ' && !isalnum(c) && c != '{' && c != '}') {
+			i_printf(MSG_EmojiBadChar, c, lineno);
+			goto fail;
+		}
+		ejbase[j++] = c;
+		state = 0;
+	}
+	ejbase[j] = 0; // null terminate
+
+	s = ejbase, lineno = 0;
+	while(++lineno, *s) {
+		bool limitprint = false;
+		long uc;
+		t = strchr(s, '\n');
+		if(t) *t++ = 0; else t = s + strlen(s);
+		if(!*s) { // empty line
+			s = t; continue;
+		}
+		if(!g) {
+// need to start a new group
+			for(u = s; isalnum(*u); ++u)  ;
+			if(u == s || !stringEqual(u, " {")) {
+				i_printf(MSG_EmojiSyntax, lineno);
+				s = t; continue;
+			}
+			g = allocMem(sizeof(struct EJGROUP));
+			g->name = s, *u = 0;
+			g->n = 0;
+			g->next = 0;
+			if(!ejgroup0) {
+				ejgroup0 = g;
+			} else {
+				struct EJGROUP *g1 = ejgroup0;
+				while(g1->next) g1 = g1->next;
+				g1->next = g;
+			}
+			limitprint = false;
+			s = t; continue;
+		}
+// now in a group
+		if(stringEqual(s, "}")) {
+			g = 0, s = t; continue;
+		}
+		uc = strtol(s, &u, 16);
+		if(uc <= 0) {
+			i_printf(MSG_EmojiUnicode, lineno);
+			s = t; continue;
+		}
+		if(!*u) { // no description
+			s = t; continue;
+		}
+		if(*u != ' ' || strchr(u, '{') || strchr(u, '}')) {
+			i_printf(MSG_EmojiSyntax, lineno);
+			s = t; continue;
+		}
+		if(g->n == EJGROUPSIZE) {
+			if(!limitprint)
+				i_printf(MSG_EmojiOverGroup, g->name, EJGROUPSIZE);
+			limitprint = true;
+		} else {
+			g->desc[g->n] = u+1;
+			g->code[g->n] = uc;
+			++g->n;
+		}
+		s = t;
+	}
+
+	return;
+
+fail:
+	free(ejbase);
+	ejbase = 0;
+}
+
+static const struct EJGROUP *ejGroupSearch(const char *s, int len)
+{
+	const struct EJGROUP *g, *g1 = 0;
+	for(g = ejgroup0; g; g = g->next) {
+		if(memcmp(g->name, s, len))
+			continue;
+		if(!g->name[len]) // exact match
+			return g;
+		if(g1) // ambiguous
+			goto fail;
+		g1 = g;
+	}
+	if(g1)
+		return g1;
+fail:
+	i_printf(g1 ? MSG_Many : MSG_NoMatch);
+	printf(": ");
+	while(len--)
+		printf("%c", *s++);
+	nl();
+	for(g = ejgroup0; g; g = g->next)
+		  printf("%s%c", g->name, (g->next ? ' ' : '\n'));
+	return 0;
+}
+
+// Gather an input line to select an emoji. Rather like the routine
+// in buffers.c, but I can't call that, or I'm infinitely recursive.
+static char *ejline; // alocated
+static int ejcommas; // selecting multiple emojis
+static void ejGetLine(void)
+{
+	char c, line[MAXTTYLINE];
+	int i, j, len, state;
+
+	ejline = 0;
+top:
+	intFlag = false;
+
+	inInput = true;
+	while (fgets(line, sizeof(line), stdin)) {
+// A bug in my keyboard causes nulls to be entered from time to time.
+		c = 0;
+		i = 0;
+		while ((unsigned)i < sizeof(line) - 1 && (c = line[i]) != '\n') {
+			if (c == 0)
+				line[i] = ' ';
+			++i;
+		}
+		if (ejline) {
+			len = strlen(ejline);
+// with nulls transliterated, strlen() returns the right answer
+			i = strlen(line);
+			ejline = reallocMem(ejline, len + i + 1);
+			strcpy(ejline + len, line);
+		} else
+			ejline = cloneString(line);
+		if (c == '\n')
+			goto tty_complete;
+	}
+	if (intFlag)
+		goto top;
+	i_puts(MSG_EndFile);
+		ebClose(1);
+
+tty_complete:
+	inInput = false;
+	state = 2, ejcommas = 0;
+	for(i = j = 0; (c = ejline[i]); ++i) {
+		if(isspace(c)) c = ' ';
+		if(c == ' ') {
+			if(!state) ejline[j++] = c, state = 1;
+			continue;
+		}
+		if(c == ',') {
+			if(state == 1) --j;
+			ejline[j++] = c;
+			state = 2, ++ejcommas;
+			continue;
+		}
+// all the whitespace has been dealt with
+		if((signed char)c <= ' ')
+			continue;
+		ejline[j++] = c;
+		state = 0;
+	}
+	if(state == 1) --j;
+ejline[j] = 0; // null terminate
+
+// take out empty fields
+	state = 2;
+	for(i = j = 0; (c = ejline[i]); ++i) {
+		if(c == ',') {
+			if(state == 2) { --ejcommas; continue; }
+			state = 2;
+		} else state = 1;
+		ejline[j++] = c;
+	}
+	if(j && line[j - 1] == ',') --j, --ejcommas;
+	ejline[j] = 0;
+}
+
+static bool ejmatches[EJGROUPSIZE];
+static int ejcount;
+
+/*********************************************************************
+Does the string you typed match one and only one emoji in the group?
+If it is from input, then you can't include spaces, so I skate
+past spaces in the matching algorithm.
+If you are selecting from a menu, then you can and should include spaces.
+Note the frommenu parameter below.
+I use to search for any substring in the string, as we do with an
+html dropdown menu, but in this case I think we want the initial substring.
+anim.b gives bear, bat, boar, etc.
+So I match what you typed against the beginning of the emoji text.
+*********************************************************************/
+
+static int ejSelect(const struct EJGROUP *g, const char *s, int len, bool frommenu)
+{
+	int partial = -1;
+	int n, i, j;
+	const char *t;
+
+// select an option by number
+	if(frommenu &&
+	(n = strtol(s, (char**)&t, 10)) >= 0 &&
+	t - s == len) {
+		if(!n || n > g->n || !ejmatches[n - 1]) {
+			i_printf(MSG_XOutOfRange, n), nl();
+			return -2;
+		}
+		return n - 1;
+	}
+
+	if(!frommenu)
+		memset(ejmatches, false, EJGROUPSIZE);
+	ejcount = 0;
+
+// search by string or substring
+	for(n = 0; n < g->n; ++n) {
+		if(frommenu && !ejmatches[n])
+			continue;
+		t = g->desc[n]; // shorthand
+		if(*t != *s)
+			continue;
+		for(i = j = 0; i < len; ++i, ++j) {
+// from the inputline we don't have spaces
+			if(t[j] == ' ' && !frommenu) {
+			--i;
+				continue;
+			}
+			if(s[i] != t[j])
+				break;
+		}
+		if(i < len) // no match
+			continue;
+		if(t == g->desc[n] && !t[j]) // exact match
+			return n;
+		if(!ejcount++) partial = n;
+		ejmatches[n] = true;
+	}
+
+	return (ejcount == 1 ? partial : -1);
+}
+
+char *selectEmoji(const char *p, int len)
+{
+	int dot; // index where dot is found
+	const struct EJGROUP *g;
+	int n;
+	char *s, *t, *ejresponse;
+
+	for(dot = 0; dot < len; ++dot)
+		if(p[dot] == '.') break;
+	if(dot == len)
+		dot = 0;
+	if(dot == len - 1)
+		dot = 0, --len;
+
+	g = ejGroupSearch(p, (dot ? dot : len));
+	if(!g || g->n == 0) // no group selected or no options
+		return 0;
+
+	if(dot) { // foo.bar
+		n = ejSelect(g, p + dot + 1, len - dot - 1, false);
+		if(n >= 0) {
+			ejresponse = allocMem(7);
+			strcpy(ejresponse, uni2utf8(g->code[n]));
+			return ejresponse;
+		}
+	}
+
+// present the menu.
+// If there were no matches on the string you provided, then present all options.
+	if(!dot || ejcount == 0)
+		memset(ejmatches, true, EJGROUPSIZE);
+	for(n = 0; n < g->n; ++n) {
+		if(ejmatches[n])
+			printf("%d: %s\n", n + 1, g->desc[n]);
+	}
+
+top:
+	ejGetLine();
+	if(!ejline[0] || stringEqual(ejline, "qt")) { // abort
+		free(ejline);
+		return 0;
+	}
+// picking one or more emojis
+	ejresponse = allocMem(6 * ejcommas + 6 + 1);
+	ejresponse[0] = 0;
+	s = ejline;
+	while(*s) {
+		t = strchr(s, ',');
+		if(t) *t++ = 0; else t = s + strlen(s);
+		n = ejSelect(g, s, strlen(s), true);
+		if(n < 0) {
+			if(n == -1)
+				i_printf(MSG_OptStartNone + (ejcount > 0), s), nl();
+			free(ejline);
+			free(ejresponse);
+			goto top;
+		}
+		strcat(ejresponse, uni2utf8(g->code[n]));
+		s = t;
+	}
+// all good
+	free(ejline);
+	return ejresponse;
+}
+
