@@ -12,6 +12,8 @@
 #endif
 #include <time.h>
 
+bool curlActive;
+int redirect_count = 0;
 char *serverData;
 int serverDataLen;
 CURL *global_http_handle;
@@ -33,6 +35,95 @@ static struct listHead down_jobs = {
 	&down_jobs, &down_jobs
 };
 
+/*
+ * Libcurl allows some really fine-grained access to data.  We could
+ * have multiple mutexes if we want, and that might lead to less
+ * blocking.  For now, we just use one mutex.
+ */
+
+static pthread_mutex_t share_mutex = PTHREAD_MUTEX_INITIALIZER;
+
+static void lock_share(CURL * handle, curl_lock_data data,
+		       curl_lock_access access, void *userptr)
+{
+/* TODO error handling. */
+	pthread_mutex_lock(&share_mutex);
+}
+
+static void unlock_share(CURL * handle, curl_lock_data data, void *userptr)
+{
+	pthread_mutex_unlock(&share_mutex);
+}
+
+void eb_curl_global_init(void)
+{
+	const unsigned int major = 7;
+	const unsigned int minor = 29;
+	const unsigned int patch = 0;
+	const unsigned int least_acceptable_version =
+	    (major << 16) | (minor << 8) | patch;
+	curl_version_info_data *version_data = NULL;
+	CURLcode curl_init_status = curl_global_init(CURL_GLOBAL_ALL);
+	if (curl_init_status != 0)
+		goto libcurl_init_fail;
+	version_data = curl_version_info(CURLVERSION_NOW);
+	if (version_data->version_num < least_acceptable_version)
+		i_printfExit(MSG_CurlVersion, major, minor, patch);
+
+// Initialize the global handle, to manage the cookie space.
+	global_share_handle = curl_share_init();
+	if (global_share_handle == NULL)
+		goto libcurl_init_fail;
+
+	curl_share_setopt(global_share_handle, CURLSHOPT_LOCKFUNC, lock_share);
+	curl_share_setopt(global_share_handle, CURLSHOPT_UNLOCKFUNC,
+			  unlock_share);
+	curl_share_setopt(global_share_handle, CURLSHOPT_SHARE,
+			  CURL_LOCK_DATA_COOKIE);
+	curl_share_setopt(global_share_handle, CURLSHOPT_SHARE,
+			  CURL_LOCK_DATA_DNS);
+	curl_share_setopt(global_share_handle, CURLSHOPT_SHARE,
+			  CURL_LOCK_DATA_SSL_SESSION);
+
+	global_http_handle = curl_easy_init();
+	if (global_http_handle == NULL)
+		goto libcurl_init_fail;
+	if (cookieFile && !ismc) {
+		curl_init_status =
+		    curl_easy_setopt(global_http_handle, CURLOPT_COOKIEFILE,
+				     "");
+		if (curl_init_status != CURLE_OK) {
+			goto libcurl_init_fail;
+		}
+		curl_init_status =
+		    curl_easy_setopt(global_http_handle, CURLOPT_COOKIEJAR,
+				     cookieFile);
+		if (curl_init_status != CURLE_OK)
+			goto libcurl_init_fail;
+	}
+	curl_init_status =
+	    curl_easy_setopt(global_http_handle, CURLOPT_ENCODING, "");
+	if (curl_init_status != CURLE_OK)
+		goto libcurl_init_fail;
+
+	curl_init_status =
+	    curl_easy_setopt(global_http_handle, CURLOPT_SHARE,
+			     global_share_handle);
+	if (curl_init_status != CURLE_OK)
+		goto libcurl_init_fail;
+	curlActive = true;
+	return;
+
+libcurl_init_fail:
+	i_printfExit(MSG_LibcurlNoInit);
+}
+
+void eb_curl_global_cleanup(void)
+{
+	curl_easy_cleanup(global_http_handle);
+	curl_global_cleanup();
+}
+
 static void setup_download(struct i_get *g);
 static CURL *http_curl_init(struct i_get *g);
 static size_t curl_header_callback(char *header_line, size_t size, size_t nmemb,
@@ -86,7 +177,7 @@ static char *find_http_header(struct i_get *g, const char *name)
 	}
 
 	return NULL;
-}				/* find_http_header */
+}
 
 static void scan_http_headers(struct i_get *g, bool fromCallback)
 {
@@ -107,7 +198,7 @@ static void scan_http_headers(struct i_get *g, bool fromCallback)
 	}
 
 	if (!g->cdfn && (v = find_http_header(g, "content-disposition"))) {
-		char *s = strstrCI(v, "filename=");
+		char *s = strcasestr(v, "filename=");
 		if (s && !strncmp(v, "attachment", 10)) {
 			s += 9;
 			if (*s == '"') {
@@ -166,7 +257,7 @@ static void scan_http_headers(struct i_get *g, bool fromCallback)
 	}
 	if (!g->auth_realm[0] && (v = find_http_header(g, "WWW-Authenticate"))) {
 		char *realm, *end;
-		if ((realm = strstrCI(v, "realm="))) {
+		if ((realm = strcasestr(v, "realm="))) {
 			realm += 6;
 			if (realm[0] == '"' || realm[0] == '\'') {
 				end = strchr(realm + 1, realm[0]);
@@ -176,7 +267,7 @@ static void scan_http_headers(struct i_get *g, bool fromCallback)
 				end = strchr(realm, ' ');
 			}
 			if (end) {
-				int sz = end - realm;
+				unsigned sz = end - realm;
 				if (sz > sizeof(g->auth_realm) - 1)
 					sz = sizeof(g->auth_realm) - 1;
 				memcpy(g->auth_realm, realm, sz);
@@ -209,7 +300,7 @@ static void scan_http_headers(struct i_get *g, bool fromCallback)
 		}
 		nzFree(v);
 	}
-}				/* scan_http_headers */
+}
 
 static void i_get_free(struct i_get *g, bool nodata)
 {
@@ -243,7 +334,7 @@ static CURLcode fetch_internet(struct i_get *g)
 	if (g->is_http)
 		scan_http_headers(g, false);
 	return curlret;
-}				/* fetch_internet */
+}
 
 /* Callback used by libcurl. Captures data from http, ftp, pop3, gopher.
  * download states:
@@ -291,7 +382,7 @@ eb_curl_callback(char *incoming, size_t size, size_t nitems, struct i_get * g)
 
 	if (g->down_state == 2 || g->down_state == 4) {	/* to disk */
 		rc = write(g->down_fd, incoming, num_bytes);
-		if (rc == num_bytes) {
+		if ((unsigned)rc == num_bytes) {
 			if (g->down_state == 4) {
 #if 0
 // Deliberately delay background download, to get several running in parallel
@@ -365,7 +456,7 @@ curl_progress(void *data_p, double dl_total, double dl_now,
 		ret = 1;
 	}
 	return ret;
-}				/* curl_progress */
+}
 
 static void
 unpackUploadedFile(const char *post, const char *boundary,
@@ -412,7 +503,7 @@ unpackUploadedFile(const char *post, const char *boundary,
 	b1 += strlen(b1);
 	*postb = post2;
 	*postb_l = b1 - post2;
-}				/* unpackUploadedFile */
+}
 
 // Date format is:    Mon, 03 Jan 2000 21:29:33 GMT|[+-]nnnn
 			// Or perhaps:     Sun Nov  6 08:49:37 1994
@@ -619,7 +710,7 @@ f4:
 fail:
 	debugPrint(3, "parseHeaderDate fails on %s", date0);
 	return 0;
-}				/* parseHeaderDate */
+}
 
 bool parseRefresh(char *ref, int *delay_p)
 {
@@ -659,7 +750,7 @@ bool parseRefresh(char *ref, int *delay_p)
 	i_printf(MSG_GarbledRefresh, ref);
 	*delay_p = 0;
 	return false;
-}				/* parseRefresh */
+}
 
 bool shortRefreshDelay(const char *r, int d)
 {
@@ -668,7 +759,7 @@ bool shortRefreshDelay(const char *r, int d)
 		return true;
 	i_printf(MSG_RedirectDelayed, r, d);
 	return false;
-}				/* shortRefreshDelay */
+}
 
 // encode the url, if it was supplied by the user.
 // Otherwise just make a copy.
@@ -680,7 +771,9 @@ static void urlSanitize(struct i_get *g, const char *post)
 
 	if (g->uriEncoded && !looksPercented(url, post)) {
 		debugPrint(2, "Warning, url %s doesn't look encoded", url);
-		g->uriEncoded = false;
+// I'm gonna trust the flag, it generally comes from running html,
+// and it's generally right, and I'm generally wrong.
+//		g->uriEncoded = false;
 	}
 
 	if (!g->uriEncoded) {
@@ -708,7 +801,7 @@ static void urlSanitize(struct i_get *g, const char *post)
 		strmove((char *)portloc, s);
 		g->urlcopy_l = strlen(g->urlcopy);
 	}
-}				/* urlSanitize */
+}
 
 bool httpConnect(struct i_get *g)
 {
@@ -719,7 +812,6 @@ bool httpConnect(struct i_get *g)
 	char *referrer = NULL;
 	CURLcode curlret = CURLE_OK;
 	struct curl_slist *custom_headers = NULL;
-	struct curl_slist *tmp_headers = NULL;
 	const struct MIMETYPE *mt;
 	char creds_buf[MAXUSERPASS * 2 + 2];	/* creds abr. for credentials */
 	bool still_fetching = true;
@@ -729,12 +821,13 @@ bool httpConnect(struct i_get *g)
 	int postb_l = 0;
 	bool transfer_status = false;
 	bool proceed_unauthenticated = false;
-	int redirect_count = 0;
 	bool post_request = false;
 	bool head_request = false;
 	uchar sxfirst = 0;
 	int n;
 
+		redirect_count = 0;
+
 	if (!getProtHostURL(url, prot, host)) {
 // only the foreground http thread uses setError,
 // the traditional /bin/ed error system.
@@ -750,6 +843,7 @@ bool httpConnect(struct i_get *g)
 		char *f;
 		urlSanitize(g, 0);
 mimestream:
+		redirect_count = 0;
 // don't have to fetch the data, the program can handle it.
 		nzFree(g->buffer);
 		g->buffer = 0;
@@ -814,11 +908,14 @@ mimestream:
 		return false;
 	}
 
+// If we just accept */*, some sites don't work:   https://crates.io
+	custom_headers = curl_slist_append(custom_headers, "Accept: text/html,application/xhtml+xml,*/*;q=0.8");
+	if (custom_headers == NULL)
+		i_printfExit(MSG_NoMem);
 /* "Expect:" header causes some servers to lose.  Disable it. */
-	tmp_headers = curl_slist_append(custom_headers, "Expect:");
-	if (tmp_headers == NULL)
+	custom_headers = curl_slist_append(custom_headers, "Expect:");
+	if (custom_headers == NULL)
 		i_printfExit(MSG_NoMem);
-	custom_headers = tmp_headers;
 	if (httpLanguage) {
 		custom_headers =
 		    curl_slist_append(custom_headers, httpLanguage);
@@ -846,17 +943,25 @@ mimestream:
 			s = strchr(post, '\r');
 			stringAndBytes(&multipart_header, &multipart_header_len,
 				       post, s - post);
-			tmp_headers =
+			custom_headers =
 			    curl_slist_append(custom_headers, multipart_header);
-			if (tmp_headers == NULL)
+			if (custom_headers == NULL)
 				i_printfExit(MSG_NoMem);
-			custom_headers = tmp_headers;
-			/* curl_slist_append made a copy of multipart_header. */
+			// curl_slist_append made a copy of multipart_header.
 			nzFree(multipart_header);
 			memcpy(thisbound, post, s - post);
 			thisbound[s - post] = 0;
 			post = s + 2;
 			unpackUploadedFile(post, thisbound, &postb, &postb_l);
+		} else if(strchr(post, '\n')) {
+// newlines indicate plain text, not url encoded.
+// But if some other content-type is specified, don't override.
+			if(!g->custom_h || !strcasestr(g->custom_h, "content-type:")) {
+				custom_headers =
+				    curl_slist_append(custom_headers, "Content-Type: text/plain");
+				if (custom_headers == NULL)
+					i_printfExit(MSG_NoMem);
+			}
 		}
 		curlret = curl_easy_setopt(h, CURLOPT_POSTFIELDS,
 					   (postb_l ? postb : post));
@@ -864,7 +969,7 @@ mimestream:
 			goto curl_fail;
 		curlret =
 		    curl_easy_setopt(h, CURLOPT_POSTFIELDSIZE,
-				     postb_l ? postb_l : strlen(post));
+				     postb_l ? (unsigned)postb_l : strlen(post));
 		if (curlret != CURLE_OK)
 			goto curl_fail;
 	} else {
@@ -925,10 +1030,9 @@ mimestream:
 			w = allocMem(d + 1);
 			memcpy(w, u, d);
 			w[d] = 0;
-			tmp_headers = curl_slist_append(custom_headers, w);
-			if (tmp_headers == NULL)
+			custom_headers = curl_slist_append(custom_headers, w);
+			if (custom_headers == NULL)
 				i_printfExit(MSG_NoMem);
-			custom_headers = tmp_headers;
 			debugPrint(4, "custom %s", w);
 			nzFree(w);
 			u = v + 1;
@@ -1164,8 +1268,11 @@ they go where they go, so this doesn't come up very often.
 		if (allowRedirection &&
 		    ((g->code >= 301 && g->code <= 303) ||
 		     (g->code >= 307 && g->code <= 308))) {
-			if (redir)
-				redir = resolveURL(g->urlcopy, redir);
+			if (redir) {
+				char *new_redir = resolveURL(g->urlcopy, redir);
+				nzFree(redir);
+				redir = new_redir;
+			}
 			still_fetching = false;
 			if (redir == NULL) {
 				/* Redirected, but we don't know where to go. */
@@ -1337,7 +1444,7 @@ curl_fail:
 	i_get_free(g, false);
 	g->referrer = referrer;
 	return transfer_status;
-}				/* httpConnect */
+}
 
 static int tsn;			// thread sequence number
 
@@ -1383,10 +1490,11 @@ void *httpConnectBack2(void *ptr)
 	g.tsn = ++tsn;
 	debugPrint(3, "jsbg thread %d", tsn);
 	rc = httpConnect(&g);
+	nzFree(g.referrer);
+	nzFree(g.cfn);
 	t->loadsuccess = rc;
-	if (!rc)
-		t->hcode = g.code;
-	else {
+	t->hcode = g.code;
+	if (rc) {
 // Rarely, a js file is not in utf8; convert it here, inside the thread.
 		char *b = force_utf8(g.buffer, g.length);
 		if (!b)
@@ -1410,7 +1518,7 @@ void *httpConnectBack3(void *ptr)
 	g.thisfile = cf->fileName;
 	g.uriEncoded = true;
 	g.url = t->href;
-	g.custom_h = t->innerHTML;
+	g.custom_h = t->custom_h;
 	g.headers_p = &outgoing_headers;
 	g.down_force = 2;
 	g.tsn = ++tsn;
@@ -1418,9 +1526,8 @@ void *httpConnectBack3(void *ptr)
 	rc = httpConnect(&g);
 	outgoing_body = g.buffer;
 	t->loadsuccess = rc;
-	if (!rc)
 		t->hcode = g.code;
-	else {
+	if(rc) {
 		char *a;
 		int l;
 // don't know why t->value would be anything
@@ -1434,16 +1541,20 @@ void *httpConnectBack3(void *ptr)
 		stringAndString(&a, &l, "\r\n\r\n");
 		stringAndNum(&a, &l, g.code);
 		stringAndString(&a, &l, "\r\n\r\n");
+		stringAndString(&a, &l, g.cfn ? g.cfn : t->href);
+		stringAndString(&a, &l, "\r\n\r\n");
 		stringAndString(&a, &l, outgoing_headers);
 		stringAndString(&a, &l, outgoing_body);
 		while (l && isspace(a[l - 1]))
 			a[--l] = 0;
 		t->value = a;
 	}
+	nzFree(g.referrer);
+	nzFree(g.cfn);
 	nzFree(outgoing_headers);
 	nzFree(outgoing_body);
-	nzFree(t->innerHTML);
-	t->innerHTML = 0;
+	nzFree(t->custom_h);
+	t->custom_h = 0;
 	return NULL;
 }
 
@@ -1547,7 +1658,7 @@ static void ftp_ls_line(struct i_get *g, char *line)
 
 	prepHtmlString(g, line);
 	stringAndChar(&g->buffer, &g->length, '\n');
-}				/* ftp_ls_line */
+}
 
 /* ftp_listing: convert an FTP-style listing to html. */
 /* Repeatedly calls ftp_ls_line to parse each line of the data. */
@@ -1576,7 +1687,7 @@ static void ftp_listing(struct i_get *g)
 
 	stringAndString(&g->buffer, &g->length, "</body></html>\n");
 	nzFree(incomingData);
-}				/* ftp_listing */
+}
 
 /* Format a line from a gopher directory. */
 static void gopher_ls_line(struct i_get *g, char *line)
@@ -1689,7 +1800,7 @@ static void gopher_ls_line(struct i_get *g, char *line)
 		prepHtmlString(g, s);
 	}
 	stringAndChar(&g->buffer, &g->length, '\n');
-}				/* gopher_ls_line */
+}
 
 /* gopher_listing: convert a gopher-style listing to html. */
 /* Repeatedly calls gopher_ls_line to parse each line of the data. */
@@ -1718,7 +1829,7 @@ static void gopher_listing(struct i_get *g)
 
 	stringAndString(&g->buffer, &g->length, "</body></html>\n");
 	nzFree(incomingData);
-}				/* gopher_listing */
+}
 
 // action: 0 traditional set, 1 print, 2 print and exit
 void ebcurl_setError(CURLcode curlret, const char *url, int action,
@@ -1826,7 +1937,7 @@ void ebcurl_setError(CURLcode curlret, const char *url, int action,
 		nl();
 	if (action == 2)
 		exit(2);
-}				/* ebcurl_setError */
+}
 
 /* Like httpConnect, but for ftp */
 static bool ftpConnect(struct i_get *g, char *creds_buf)
@@ -1970,7 +2081,7 @@ ftp_transfer_fail:
 	i_get_free(g, !transfer_success);
 
 	return transfer_success;
-}				/* ftpConnect */
+}
 
 /* Like httpConnect, but for gopher */
 static bool gopherConnect(struct i_get *g)
@@ -2110,7 +2221,7 @@ gopher_transfer_fail:
 	}
 
 	return true;
-}				/* gopherConnect */
+}
 
 /* If the user has asked for locale-specific responses, then build an
  * appropriate Accept-Language: header. */
@@ -2133,7 +2244,7 @@ void setHTTPLanguage(const char *lang)
 	for (s = httpLanguage; *s; ++s)
 		if (*s == '_')
 			*s = '-';
-}				/* setHTTPLanguage */
+}
 
 /* Set the FD_CLOEXEC flag on a socket newly-created by libcurl.
  * Let's not leak libcurl's sockets to child processes created by the
@@ -2237,7 +2348,7 @@ libcurl_init_fail:
 	if (h)
 		curl_easy_cleanup(h);
 	return 0;
-}				/* http_curl_init */
+}
 
 /*
  * There's no easy way to get at the server's response message from libcurl.
@@ -2252,7 +2363,7 @@ static const char *response_codes_1xx[] = {
 
 static const char *response_codes_2xx[] = {
 	"OK",
-	"Created" "Accepted",
+	"Created", "Accepted",
 	"Non-Authoritative Information",
 	"No Content",
 	"Reset Content",
@@ -2331,7 +2442,7 @@ static const char *message_for_response_code(int code)
 			message = responses[primary][subcode];
 	}
 	return message;
-}				/* message_for_response_code */
+}
 
 /*
  * Function: prompt_and_read
@@ -2377,7 +2488,7 @@ prompt_and_read(int prompt, char *buffer, int buffer_length, int error_message,
 			reading = false;
 	}
 	return n;
-}				/* prompt_and_read */
+}
 
 /*
  * Function: read_credentials
@@ -2421,7 +2532,7 @@ static bool read_credentials(char *buffer)
 	}
 
 	return got_creds;
-}				/* read_credentials */
+}
 
 /* Callback used by libcurl.
  * Gather all the http headers into one long string. */
@@ -2455,7 +2566,7 @@ curl_header_callback(char *header_line, size_t size, size_t nmemb,
 	}
 
 	return bytes_in_line;
-}				/* curl_header_callback */
+}
 
 /* Print text, discarding the unnecessary carriage return character. */
 static void
@@ -2466,7 +2577,31 @@ prettify_network_text(const char *text, size_t size, FILE * destination)
 		if (text[i] != '\r')
 			fputc(text[i], destination);
 	}
-}				/* prettify_network_text */
+}
+
+static void
+prettify_network_data(const char *s, size_t size, FILE * destination)
+{
+	size_t i;
+	if(size > 4000) {
+		printf("length %zu\n", size);
+		return;
+	}
+
+// if it happens to be ascii then roll with that.
+	for(i = 0; i < size; ++i)
+		if((signed char)s[i] <= 0)
+			goto nonascii;
+	prettify_network_text(s, size, destination);
+	if(size && s[size - 1] != '\n')
+	fprintf(destination, "\n");
+	return;
+
+nonascii:
+	for(i = 0; i < size; ++i)
+		fprintf(destination, "%02x", (uchar)s[i]);
+	fprintf(destination, "\n");
+}
 
 /* Print incoming and outgoing headers.
  * Incoming headers are prefixed with curl<, and outgoing headers are
@@ -2479,6 +2614,7 @@ ebcurl_debug_handler(CURL * handle, curl_infotype info_desc, char *data,
 {
 	FILE *f = debugFile ? debugFile : stdout;
 
+
 // There's a special case where this function is used
 // by the imap client to see if the server is move capable.
 	if (ismc & isimap && info_desc == CURLINFO_HEADER_IN &&
@@ -2494,6 +2630,7 @@ ebcurl_debug_handler(CURL * handle, curl_infotype info_desc, char *data,
 	if (debugLevel < 4)
 		return 0;
 
+// incoming headers seem to come in several small bursts.
 	if (info_desc == CURLINFO_HEADER_OUT) {
 		fprintf(f, "curl>\n");
 		prettify_network_text(data, size, f);
@@ -2501,7 +2638,10 @@ ebcurl_debug_handler(CURL * handle, curl_infotype info_desc, char *data,
 		if (!g->last_curlin)
 			fprintf(f, "curl<\n");
 		prettify_network_text(data, size, f);
-	} else;			/* Do nothing.  We don't care about this piece of data. */
+	} else if (info_desc == CURLINFO_DATA_OUT) {
+			fprintf(f, "curl+\n");
+		prettify_network_data(data, size, f);
+	}
 
 	if (info_desc == CURLINFO_HEADER_IN)
 		g->last_curlin = true;
@@ -2509,7 +2649,7 @@ ebcurl_debug_handler(CURL * handle, curl_infotype info_desc, char *data,
 		g->last_curlin = false;
 
 	return 0;
-}				/* ebcurl_debug_handler */
+}
 
 // At this point, down_state = 1
 // Only runs from the foreground thread, does not have to be threadsafe.
@@ -2577,7 +2717,7 @@ top:
 	}
 
 	g->down_state = (down_bg ? 5 : 2);
-}				/* setup_download */
+}
 
 /* show background jobs and return the number of jobs pending */
 /* if iponly is true then just show in progress */
@@ -2669,14 +2809,108 @@ CURLcode setCurlURL(CURL * h, const char *url)
 	if (sslCerts)
 		curl_easy_setopt(h, CURLOPT_CAINFO, sslCerts);
 	return curl_easy_setopt(h, CURLOPT_URL, url);
-}				/* setCurlURL */
+}
+
+/*********************************************************************
+I'm putting the frame expand stuff here cause there really isn't a good place
+to put it, and it sort of goes with http access.
+frameExpand(expand, start, end)
+Pass a range of lines; you can expand all the frames in one go.
+Return false if there is a problem fetching a web page,
+or if none of the lines are frames.
+If first argument expand is false then we are contracting.
+Call either frameExpandLine or frameContractLine on each line in the range.
+frameExpandLine takes a line number or a tag, not both.
+One or the other will be 0.
+If a line number, it comes from a user command, you asked to expand the frame.
+If the tag is not null, it is from a getter,
+javascript is trying to access objects within that frame,
+and now we need to expand it on demand.
+crossOrigin() checks for a violation, wherein an evil website tries
+to bring your bank or paypal or something valuable in as a frame, then dip into
+its objects to read your password or other critical data.
+Return true if it's ok to procede, however,
+I may put an Origin: header on the tag if such is needed.
+I don't know the actual criteria, I'm just guessing. I'm starting with:
+https://en.wikipedia.org/wiki/Cross-origin_resource_sharing
+*********************************************************************/
+
+static bool crossOrigin(Tag *t, const char *url2)
+{
+	const char *url1 = t->f0->hbase;
+	bool u1, u2;
+	const char *host1, *host2;
+
+// If the bottom url is empty then it is a blank frame, which is fine,
+// or it was some javascript code, which has been moved to another
+// variable, and that's fine.
+	if(!url2)
+		return true;
+
+// If the top url is null then it is a local file with no filename,
+// so unlikely that I'm not gonna worry about it, or it's a blank frame.
+// A blank frame pulling in internet frames below it?
+// I don't know what to make of that, so I don't trust it.
+	if(!url1) {
+		debugPrint(3, "crossorigin violation: blank over internet");
+		return false;
+	}
+
+	u1 = isURL(url1);
+	u2 = isURL(url2);
+// if the top is a local file, not a url, then you are starting with a base html
+// file on your home computer. You are responsible for this file.
+// You can reference other local files or urls as frames.
+	if(!u1)
+		return true;
+
+// I don't know how the top could be a url and the bottom a local file.
+// The bottom would be resolved against the top, so an attacker
+// who built a wnasty website on example.com, and then tried to access
+// your .ssh/id_rsa file through a frame would actually get
+// example.com/.ssh/id_rsa.
+// That's just another file on example.com.
+// Be that as it may, the result of such an attack would be so hideous,
+// that I'll guard against it here.
+	if(!u2) {
+		debugPrint(3, "crossorigin violation: internet over local");
+		return false;
+	}
+
+// they are both urls, compare the domains.
+// Neither of them should come out null, but I'll check for that.
+	if(!(host1 = getHostURL(url1))) {
+		debugPrint(3, "crossorigin: top %s unrecognized domain", url1);
+		return false;
+	}
+	host1 = cloneString(host1);
+	if(!(host2 = getHostURL(url2))) {
+		debugPrint(3, "crossorigin: bottom %s unrecognized domain", url2);
+		cnzFree(host1);
+		return false;
+	}
+
+// Compare the two domains. I'm using a routine from the cookie
+// world, assuming the criteria are the same.
+	if(!isInDomain(host1, host2)) {
+		char *orig_head;
+		debugPrint(3, "crossorigin attempt: %s over %s", host1, host2);
+// We need to send origin as an http header.
+// Should it be the domain, or the entire url?
+// The wiki simple minimal example includes the protocol, so I'll go with url.
+		cnzFree(host1);
+		orig_head = allocMem(strlen(url1) + 10);
+		sprintf(orig_head, "Origin: %s\n", url1);
+		t->custom_h = orig_head;
+		return true;
+	}
+
+	cnzFree(host1);
+	return true;
+}
+
+static int frameContractLine(int ln);
 
-/* expand a frame inline.
- * Pass a range of lines; you can expand all the frames in one go.
- * Return false if there is a problem fetching a web page,
- * or if none of the lines are frames. */
-static int frameContractLine(int lineNumber);
-static const char *stringInBufLine(const char *s, const char *t);
 bool frameExpand(bool expand, int ln1, int ln2)
 {
 	int ln;			/* line number */
@@ -2701,29 +2935,23 @@ bool frameExpand(bool expand, int ln1, int ln2)
 	if (problem == 2)
 		setError(MSG_FrameNoURL);
 	return (problem == 0);
-}				/* frameExpand */
+}
 
 /* Problems: 0, frame expanded successfully.
  1 line is not a frame.
  2 frame doesn't have a valid url.
  3 Problem fetching the rul or rendering the page.  */
-int frameExpandLine(int ln, jsobjtype fo)
+int frameExpandLine(int ln, Tag *t)
 {
 	pst line;
 	int tagno, start;
-	const char *s;
+	const char *s, *jssrc = 0;
 	char *a;
-	char *jssrc = 0;
-	Tag *t;
-	Frame *save_cf, *new_cf, *last_f;
-	uchar save_local;
+	Frame *save_cf, *last_f;
+	bool fromget = !ln;
 	Tag *cdt;	// contentDocument tag
 
-	if (fo) {
-		t = tagFromJavaVar(fo);
-		if (!t)
-			return 1;
-	} else {
+	if(!t) {
 		line = fetchLine(ln, -1);
 		s = stringInBufLine((char *)line, "Frame ");
 		if (!s)
@@ -2735,25 +2963,41 @@ int frameExpandLine(int ln, jsobjtype fo)
 			return 2;
 		t = tagList[tagno];
 	}
+
 	if (t->action != TAGACT_FRAME)
 		return 1;
 
 /* the easy case is if it's already been expanded before, we just unhide it. */
 	if (t->f1) {
-		if (!fo)
+// If js is accessing objects in this frame, that doesn't mean we unhide it.
+		if (!fromget)
 			t->contracted = false;
 		return 0;
 	}
+
+// maybe we tried to expand it and it failed
+	if(t->expf)
+		return 0;
+	t->expf = true;
+
 // Check with js first, in case it changed.
-	if (t->jv && (a = get_property_url(t->f0, t->jv, false)) && *a) {
+	if ((a = get_property_url_t(t, false)) && *a) {
 		nzFree(t->href);
 		t->href = a;
 	}
 	s = t->href;
+	if(s && !*s)
+		s = 0;
+	// we check for about:blank in the html, but frames can be
+// created by js.
+	if(stringEqual(s, "about:blank")) {
+		nzFree(t->href);
+		s = t->href = 0;
+	}
 
 // javascript in the src, what is this for?
 	if (s && !strncmp(s, "javascript:", 11)) {
-		jssrc = (char *)s;
+		jssrc = s;
 		s = 0;
 	}
 
@@ -2761,15 +3005,16 @@ int frameExpandLine(int ln, jsobjtype fo)
 // No source. If this is your request then return an error.
 // But if we're dipping into the objects then it needs to expand
 // into a separate window, a separate js space, with an empty body.
-		if (!fo && !jssrc)
+		if (!fromget && !jssrc)
 			return 2;
 // After expansion we need to be able to expand it,
 // because there's something there, well maybe.
 		t->href = cloneString("#");
-// jssrc is the old href and we are responsible for it
+// jssrc is the old javascript href and now we are responsible for it
 	}
 
 	save_cf = cf = t->f0;
+
 /* have to push a new frame before we read the web page */
 	for (last_f = &(cw->f0); last_f->next; last_f = last_f->next) ;
 	last_f->next = cf = allocZeroMem(sizeof(Frame));
@@ -2780,7 +3025,9 @@ int frameExpandLine(int ln, jsobjtype fo)
 		   (s ? s : (jssrc ? "javascript" : "empty")));
 
 	if (s) {
-		bool rc = readFileArgv(s, (fo ? 2 : 1));
+		bool rc = false;
+		if(crossOrigin(t, s))
+			rc = readFileArgv(s, (fromget ? 2 : 1), t->custom_h);
 		if (!rc) {
 /* serverData was never set, or was freed do to some other error. */
 /* We just need to pop the frame and return. */
@@ -2789,7 +3036,7 @@ int frameExpandLine(int ln, jsobjtype fo)
 			free(cf);
 			last_f->next = 0;
 			cf = save_cf;
-			return 3;
+			return 0;
 		}
 
        /*********************************************************************
@@ -2813,7 +3060,6 @@ So check for serverData null here. Once again we pop the frame.
 		serverDataLen = strlen(serverData);
 	}
 
-	new_cf = cf;
 	if (changeFileName) {
 		nzFree(cf->fileName);
 		cf->fileName = changeFileName;
@@ -2827,67 +3073,53 @@ So check for serverData null here. Once again we pop the frame.
 	fileSize = -1;
 
 /* If we got some data it has to be html.
- * I should check for that, something like htmlTest in html.c,
+ * I should check for that, something like htmlTest(),
  * but I'm too lazy to do that right now, so I'll just assume it's good.
  * Also, we have verified content-type = text/html, so that's pretty good. */
 
 	cf->hbase = cloneString(cf->fileName);
-	save_local = browseLocal;
-	browseLocal = !isURL(cf->fileName);
 	prepareForBrowse(serverData, serverDataLen);
 	if (javaOK(cf->fileName))
-		createJavaContext();
+		createJSContext(cf);
 	nzFree(newlocation);	/* should already be 0 */
 	newlocation = 0;
 
 	start = cw->numTags;
+	cdt = newTag(cf, "Document");
+	cdt->parent = t, t->firstchild = cdt;
+	cdt->attributes = allocZeroMem(sizeof(char*));
+	cdt->atvals = allocZeroMem(sizeof(char*));
+	debugPrint(3, "parse html from frame");
 /* call the tidy parser to build the html nodes */
 	html2nodes(serverData, true);
 	nzFree(serverData);	/* don't need it any more */
 	serverData = 0;
 	htmlGenerated = false;
-// in the edbrowse world, the only child of the frame tag
-// is the contentDocument tag.
-	cdt = t->firstchild;
-// the placeholder document node will soon be orphaned.
-	delete_property(cdt->f0, cdt->jv, "parentNode");
-	htmlNodesIntoTree(start, cdt);
-	cdt->step = 0;
+	htmlNodesIntoTree(start + 1, cdt);
 	prerender(0);
 
 /*********************************************************************
 At this point cdt->step is 1; the html tree is built, but not decorated.
-Well I put the object on cdt manually. Besides, we don't want to set up
-the fake cdt object and the getter that auto-expands the frame,
-we did that before and now it's being expanded. So bump step up to 2.
+cdt doesn't have or need an object; it's a place holder.
 *********************************************************************/
 	cdt->step = 2;
 
-	if (cf->docobj) {
-		jsobjtype topobj;
+	if (cf->jslink) {
 		decorate(0);
 		set_basehref(cf->hbase);
-// parent points to the containing frame.
-		set_property_object(cf, cf->winobj, "parent", save_cf->winobj);
-// And top points to the top.
-		cf = save_cf;
-		topobj = get_property_object(cf, cf->winobj, "top");
-		cf = new_cf;
-		set_property_object(cf, cf->winobj, "top", topobj);
-		set_property_object(cf, cf->winobj, "frameElement", t->jv);
-		run_function_bool(cf, cf->winobj, "eb$qs$start");
-		if (jssrc) {
-			jsRunScript(cf, cf->winobj, jssrc, "frame.src", 1);
-		}
+		run_function_bool_win(cf, "eb$qs$start");
+		if (jssrc)
+			jsRunScriptWin(jssrc, "frame.src", 1);
 		runScriptsPending(true);
 		runOnload();
 		runScriptsPending(false);
-		set_property_string(cf, cf->docobj, "readyState", "complete");
-		run_event_bool(cf, cf->docobj, "document", "onreadystatechange");
+		run_function_bool_win(cf, "readyStateComplete");
 		runScriptsPending(false);
 		rebuildSelectors();
 	}
-	nzFree(jssrc);
+	cnzFree(jssrc);
+	cf->browseMode = true;
+	debugPrint(3, "end parse html from frame");
 
 	if (cf->fileName) {
 		int j = strlen(cf->fileName);
@@ -2896,34 +3128,20 @@ we did that before and now it's being expanded. So bump step up to 2.
 	}
 
 	t->f1 = cf;
-	cf = save_cf;
-	browseLocal = save_local;
-	if (fo)
+	if (fromget)
 		t->contracted = true;
-	if (new_cf->docobj) {
-		jsobjtype cdo;	// contentDocument object
-		jsobjtype cwo;	// contentWindow object
-		jsobjtype cna;	// childNodes array
-		cdo = new_cf->docobj;
-		disconnectTagObject(cdt);
-		connectTagObject(cdt, cdo);
-		cdt->style = 0;
-// Should I switch this tag into the new frame? I don't really know.
-		cdt->f0 = new_cf;
-		set_property_object(new_cf, t->jv, "content$Document", cdo);
-		cna = get_property_object(t->f0, t->jv, "childNodes");
-		set_array_element_object(t->f0, cna, 0, cdo);
-// Should we do this? For consistency I guess yes.
-		set_property_object(t->f0, cdo, "parentNode", t->jv);
-		cwo = new_cf->winobj;
-		set_property_object(new_cf, t->jv, "content$Window", cwo);
+	if (cf->jslink) {
+		reconnectTagObject(cdt);
+		set_property_bool_t(t, "eb$expf", true);
 // run the frame onload function if it is there.
 // I assume it should run in the higher frame.
-		run_event_bool(t->f0, t->jv, t->info->name, "onload");
+		run_event_t(t, t->info->name, "onload");
 	}
 
+// success, frame is expanded
+	cf = save_cf;
 	return 0;
-}				/* frameExpandLine */
+}
 
 static int frameContractLine(int ln)
 {
@@ -2932,74 +3150,26 @@ static int frameContractLine(int ln)
 		return 1;
 	t->contracted = true;
 	return 0;
-}				/* frameContractLine */
-
-Tag *line2frame(int ln)
-{
-	const char *line;
-	int n, opentag = 0, ln1 = ln;
-	const char *s;
-
-	for (; ln; --ln) {
-		line = (char *)fetchLine(ln, -1);
-		if (!opentag && ln < ln1
-		    && (s = stringInBufLine(line, "*--`\n"))) {
-			for (--s; s > line && *s != InternalCodeChar; --s) ;
-			if (*s == InternalCodeChar)
-				opentag = atoi(s + 1);
-			continue;
-		}
-		s = stringInBufLine(line, "*`--\n");
-		if (!s)
-			continue;
-		for (--s; s > line && *s != InternalCodeChar; --s) ;
-		if (*s != InternalCodeChar)
-			continue;
-		n = atoi(s + 1);
-		if (!opentag)
-			return tagList[n];
-		if (n == opentag)
-			opentag = 0;
-	}
-
-	return 0;
-}				/* line2frame */
-
-/* a text line in the buffer isn't a string; you can't use strstr */
-static const char *stringInBufLine(const char *s, const char *t)
-{
-	int n = strlen(t);
-	for (; *s != '\n'; ++s) {
-		if (!strncmp(s, t, n))
-			return s;
-	}
-	return 0;
-}				/* stringInBufLine */
+}
 
 bool reexpandFrame(void)
 {
 	int j, start;
 	Tag *frametag;
 	Tag *cdt;	// contentDocument tag
-	uchar save_local;
+	Frame *save_cf = cf;
 	bool rc;
-	jsobjtype save_top, save_parent, save_fe;
 
+// I don't know why cf would ever not be newloc_f.
 	cf = newloc_f;
 	frametag = cf->frametag;
 	cdt = frametag->firstchild;
-	save_top = get_property_object(cf, cf->winobj, "top");
-	save_parent = get_property_object(cf, cf->winobj, "parent");
-	save_fe = get_property_object(cf, cf->winobj, "frameElement");
 
 // Cut away our tree nodes from the previous document, which are now inaccessible.
 	underKill(cdt);
 
-// the previous document node will soon be orphaned.
-	delete_property(cf, cdt->jv, "parentNode");
-
 	delTimers(cf);
-	freeJavaContext(cf);
+	freeJSContext(cf);
 	nzFree(cf->dw);
 	cf->dw = 0;
 	nzFree(cf->hbase);
@@ -3010,16 +3180,18 @@ bool reexpandFrame(void)
 	cf->uriEncoded = false;
 	nzFree(cf->firstURL);
 	cf->firstURL = 0;
-	rc = readFileArgv(cf->fileName, 2);
+	rc = readFileArgv(cf->fileName, 2, 0);
 	if (!rc) {
 /* serverData was never set, or was freed do to some other error. */
 		fileSize = -1;	/* don't print 0 */
+		cf = save_cf;
 		return false;
 	}
 
 	if (serverData == NULL) {
 /* frame replaced itself with a playable stream, what to do? */
 		fileSize = -1;
+		cf = save_cf;
 		return true;
 	}
 
@@ -3034,148 +3206,43 @@ bool reexpandFrame(void)
 	fileSize = -1;
 
 	cf->hbase = cloneString(cf->fileName);
-	save_local = browseLocal;
-	browseLocal = !isURL(cf->fileName);
 	prepareForBrowse(serverData, serverDataLen);
 	if (javaOK(cf->fileName))
-		createJavaContext();
+		createJSContext(cf);
 
 	start = cw->numTags;
+	debugPrint(3, "parse html from frame replace");
 /* call the tidy parser to build the html nodes */
 	html2nodes(serverData, true);
 	nzFree(serverData);	/* don't need it any more */
 	serverData = 0;
+	cf->browseMode = false;
 	htmlGenerated = false;
 	htmlNodesIntoTree(start, cdt);
 	cdt->step = 0;
 	prerender(0);
 	cdt->step = 2;
-	if (cf->docobj) {
+
+	if (cf->jslink) {
 		decorate(0);
 		set_basehref(cf->hbase);
-		set_property_object(cf, cf->winobj, "top", save_top);
-		set_property_object(cf, cf->winobj, "parent", save_parent);
-		set_property_object(cf, cf->winobj, "frameElement", save_fe);
-		run_function_bool(cf, cf->winobj, "eb$qs$start");
+		run_function_bool_win(cf, "eb$qs$start");
 		runScriptsPending(true);
 		runOnload();
 		runScriptsPending(false);
-		set_property_string(cf, cf->docobj, "readyState", "complete");
-		run_event_bool(cf, cf->docobj, "document", "onreadystatechange");
+		run_function_bool_win(cf, "readyStateComplete");
 		runScriptsPending(false);
 		rebuildSelectors();
 	}
+	cf->browseMode = true;
+	debugPrint(3, "end parse html from frame replace");
 
 	j = strlen(cf->fileName);
 	cf->fileName = reallocMem(cf->fileName, j + 8);
 	strcat(cf->fileName, ".browse");
-	browseLocal = save_local;
 
-	if (cf->docobj) {
-		Frame *save_cf;
-		jsobjtype cdo;	// contentDocument object
-		jsobjtype cwo;	// contentWindow object
-		jsobjtype cna;	// childNodes array
-		cdo = cf->docobj;
-		cwo = cf->winobj;
-		disconnectTagObject(cdt);
-		connectTagObject(cdt, cdo);
-		cdt->style = 0;
-// Should I switch this tag into the new frame? I don't really know.
-		cdt->f0 = cf;
-// have to point contentDocument to the new document object,
-// but that requires a change of context.
-		save_cf = cf;
-		cf = frametag->f0;
-		set_property_object(cf, frametag->jv, "content$Document", cdo);
-		cna = get_property_object(cf, frametag->jv, "childNodes");
-		set_array_element_object(cf, cna, 0, cdo);
-// Should we do this? For consistency I guess yes.
-		set_property_object(cf, cdo, "parentNode", frametag->jv);
-		set_property_object(cf, frametag->jv, "content$Window", cwo);
-		cf = save_cf;
-	}
-
-	return true;
-}				/* reexpandFrame */
-
-// Make sure a web page is not trying to read a local file.
-bool frameSecurityFile(const char *thisfile)
-{
-	Frame *f = &cf->owner->f0;
-	for (; f != cf; f = f->next) {
-		if (!isURL(f->fileName))
-			continue;
-		setError(MSG_NoAccessSecure, thisfile);
-		return false;
-	}
+	if (cf->jslink)
+		reconnectTagObject(cdt);
+	cf = save_cf;
 	return true;
 }
-
-static bool remember_contracted;
-
-// Undo the above,as though the frame were never expanded.
-void unframe(jsobjtype fobj, jsobjtype newdoc)
-{
-	int i, n;
-	Tag *t, *cdt;
-	jsobjtype cdo;
-	Frame *f, *f1;
-
-	t = tagFromJavaVar(fobj);
-	if (!t) {
-		debugPrint(1, "unframe couldn't find tag");
-		return;
-	}
-	if (!(cdt = t->firstchild) || cdt->action != TAGACT_DOC || cdt->sibling
-	    || !(cdo = cdt->jv)) {
-		debugPrint(1, "unframe child tag isn't right");
-		return;
-	}
-	underKill(cdt);
-	disconnectTagObject(cdt);
-	connectTagObject(cdt, newdoc);
-
-	f1 = t->f1;
-	t->f1 = 0;
-	remember_contracted = t->contracted;
-	if (f1 == cf) {
-		debugPrint(1,
-			   "deleting the current frame, this shouldn't happen, edbrowse is corrupt");
-		return;
-	}
-	for (f = &(cw->f0); f; f = f->next)
-		if (f->next == f1)
-			break;
-	if (!f) {
-		debugPrint(1, "unframe can't find prior frame to relink");
-		return;
-	}
-	f->next = f1->next;
-	delTimers(f1);
-	freeJavaContext(f1);
-	nzFree(f1->dw);
-	nzFree(f1->hbase);
-	nzFree(f1->fileName);
-	nzFree(f1->firstURL);
-	free(f1);
-
-// cdt use to belong to f1, which no longer exists.
-	cdt->f0 = f;		// back to its parent frame
-
-// A running frame could create nodes in its parent frame, or any other frame.
-	n = 0;
-	for (i = 0; i < cw->numTags; ++i) {
-		t = tagList[i];
-		if (t->f0 == f1)
-			t->f0 = f, ++n;
-	}
-	if (n)
-		debugPrint(3, "%d nodes pushed up to the parent frame", n);
-}
-
-void unframe2(jsobjtype fobj)
-{
-	Tag *t = tagFromJavaVar(fobj);
-	t->contracted = remember_contracted;
-}
