@@ -14,7 +14,7 @@ blank spaces for messages no longer used.
 Each file lang/msg-* must have exactly this many lines.
 *********************************************************************/
 
-#define EdbrowseMessageCount 662
+#define EdbrowseMessageCount 700
 
 // English
 extern const char *msg_en[], ebrc_en[], qrg_en[];
@@ -331,7 +331,7 @@ enum {
 	MSG_BrowseEmpty,
 	MSG_Unbrowsable,
 	MSG_BrowseAlready,
-	MSG_NoLable2,
+	MSG_NoLabel2,
 	MSG_BrowseI,
 	MSG_InsertFunction,
 	MSG_LineUpdateRange,
@@ -411,7 +411,7 @@ enum {
 	MSG_FetchN,
 	MSG_NoAttachments,
 	MSG_EBRC_KeyInFunc,
-	MSG_notused314,
+	MSG_NoTable,
 	MSG_FTPSession,
 	MSG_Push,
 	MSG_DBNotCompiled,
@@ -426,7 +426,7 @@ enum {
 	MSG_NoFolderMatch,
 	MSG_ManyFolderMatch,
 	MSG_Search,
-	MSG_Limit,
+	MSG_Envelope,
 	MSG_MoveTo,
 	MSG_SectionIgnored,
 	MSG_FileXEmpty,
@@ -703,7 +703,7 @@ enum {
 	MSG_DecodePost,
 	MSG_WrapOff,
 	MSG_WrapOn,
-	MSG_notused606,
+	MSG_NoColumnHeaders,
 	MSG_ReDir,
 	MSG_ReDB,
 	MSG_ReEmpty,
@@ -760,4 +760,22 @@ enum {
 	MSG_ColorOn,
 	MSG_ShowLast,
 	MSG_ShowFirst,
+	MSG_Previous,
+	MSG_NoPrevMail,
+	MSG_Textarea0,
+	MSG_EBRC_MailDirect,
+	MSG_OptionC,
+	MSG_FileC,
+	MSG_ReqField,
+	MSG_Limit,
+	MSG_TextRec,
+	MSG_AtSyntax,
+	MSG_EBRC_EmojiNotFile,
+	MSG_EmojiBadControl,
+	MSG_EmojiBadChar,
+	MSG_EmojiOverGroup,
+	MSG_EmojiUnicode,
+	MSG_EmojiSyntax,
+	MSG_OptStartNone,
+	MSG_OptStartMany,
 };
