@@ -17,8 +17,9 @@ static char subjectLine[400];
 static int mailAccount;
 
 static struct ALIAS {
-	char name[16];
-	char email[64];
+	char brief[16];
+	char email[100];
+	char fullname[100 + 4];
 } *addressList;
 static int nads;		/* number of addresses */
 static time_t adbooktime;
@@ -81,6 +82,7 @@ freefail:
 				goto freefail;
 			}
 			if (state == 3) {
+				bool greater = false;
 				++nads;
 				while (isspaceByte(t[-1]))
 					--t;
@@ -91,23 +93,24 @@ freefail:
 					goto freefail;
 				}
 				++v;
-				if (t - v >= 64) {
-					setError(MSG_ABMailLong, ln - 1);
+				if (t - v >= 100) {
+					setError(MSG_ABMailLong, ln - 1, 100 - 1);
 					goto freefail;
 				}
 				if (!strchr(v, '@')) {
 					setError(MSG_ABNoAt, ln - 1);
 					goto freefail;
 				}
-				if (strpbrk(v, " \t")) {
-					setError(MSG_ABMailSpaces, ln - 1);
-					goto freefail;
-				}
 
 				while (last < t) {
 					if (!isprintByte(*last)) {
-						setError(MSG_AbMailUnprintable,
-							 ln - 1);
+						setError(MSG_AbMailUnprintable, ln - 1);
+						goto freefail;
+					}
+					if(*last == '>')
+						greater = true;
+					if ((*last == ' ' || *last == '\t') && !greater && last > v) {
+						setError(MSG_ABMailSpaces, ln - 1);
 						goto freefail;
 					}
 					++last;
@@ -140,32 +143,38 @@ freefail:
 		j = 0;
 		for (s = buf; *s; s = t + 1, ++j) {
 			t = strchr(s, ':');
-			memcpy(addressList[j].name, s, t - s);
-			addressList[j].name[t - s] = 0;
+			memcpy(addressList[j].brief, s, t - s);
+			addressList[j].brief[t - s] = 0;
 			s = t + 1;
 			t = strchr(s, '\n');
 			memcpy(addressList[j].email, s, t - s);
 			addressList[j].email[t - s] = 0;
+			addressList[j].fullname[0] = 0;
+			if ((v = strchr(addressList[j].email, '>'))) {
+				*v++ = 0;
+				sprintf(addressList[j].fullname,
+				"%s <%s>", v, addressList[j].email);
+			}
 		}
 	}
-	/* aliases are present */
+
 	nzFree(buf);
 	adbooktime = mtime;
 	return true;
-}				/* loadAddressBook */
+}
 
 const char *reverseAlias(const char *reply)
 {
 	int i;
 	for (i = 0; i < nads; ++i)
 		if (stringEqual(reply, addressList[i].email)) {
-			const char *a = addressList[i].name;
+			const char *a = addressList[i].brief;
 			if (*a == '!')
 				break;
 			return a;
 		}
 	return 0;		/* not found */
-}				/* reverseAlias */
+}
 
 static char *qpEncode(const char *line)
 {
@@ -186,7 +195,7 @@ static char *qpEncode(const char *line)
 	}
 
 	return newbuf;
-}				/* qpEncode */
+}
 
 /* Return 0 if there was no need to encode */
 static char *isoEncode(char *start, char *end)
@@ -228,7 +237,7 @@ package:
 	sprintf(t, "=?ISO-8859-1?%c?%s?=", code, s);
 	nzFree(s);
 	return t;
-}				/* isoEncode */
+}
 
 /*********************************************************************
 Return a string that defines the charset of the outgoing mail.
@@ -249,7 +258,7 @@ static char *charsetString(const char *ct, const char *ce)
 			sprintf(buf, "; charset=iso-8859-%d", type8859);
 	}
 	return buf;
-}				/* charsetString */
+}
 
 /* Read a file into memory, mime encode it,
  * and return the type of encoding and the encoded data.
@@ -323,11 +332,12 @@ empty:
 		}
 		while (*s == ' ' || *s == '\t')
 			++s;
-		if (!memEqualCI(s, "subject:", 8)) {
+		if (!memEqualCI(s, "subject:", 8)
+		&& !memEqualCI(s, "sub:", 4)) {
 			setError(MSG_SubjectStart);
 			goto freefail;
 		}
-		s += 8;
+		s += (s[3] == ':' ? 4 : 8);
 		while (*s == ' ' || *s == '\t')
 			++s;
 		t = s;
@@ -336,7 +346,7 @@ empty:
 		v = s;
 		while (s > t && isspaceByte(s[-1]))
 			--s;
-		if (s - t >= sizeof(subjectLine)) {
+		if ((unsigned)(s - t) >= sizeof(subjectLine)) {
 			setError(MSG_SubjectLong, sizeof(subjectLine) - 1);
 			goto freefail;
 		}
@@ -423,6 +433,18 @@ empty:
 			ct = "text/html";
 	}
 
+// alternative from a buffer is usually html; this doesn't fly if wev
+// send it over as plain text. This is a crude test.
+// Just look for a leading <
+	if(!strncmp(file, "<buffer ", 8)) {
+		for (i = 0; i < buflen; ++i) {
+			c = buf[i];
+			if(!isspace(c)) break;
+		}
+		if(i < buflen && c == '<')
+			ct = "text/html";
+	}
+
 /* Count the nonascii characters */
 	nacount = nullcount = nlcount = 0;
 	longline = false;
@@ -431,7 +453,7 @@ empty:
 		c = buf[i];
 		if (c == '\0')
 			++nullcount;
-		if (c < 0)
+		if (c & 0x80)
 			++nacount;
 		if (c != '\n')
 			continue;
@@ -465,7 +487,7 @@ empty:
  * nulls, which is a quirk of my slapped together software. */
 
 	if ((!webform && buflen > 20 && nacount * 5 > buflen) ||
-	    (webform && nullcount)) {
+	    nullcount) {
 		if (ismail) {
 			setError(MSG_MailBinary, file);
 			goto freefail;
@@ -478,18 +500,8 @@ empty:
 	}
 
 	if (!webform) {
-/* Switch to unix newlines - we'll switch back to dos later. */
-		v = buf + buflen;
-		for (s = t = buf; s < v; ++s) {
-			c = *s;
-			if (c == '\r' && s < v - 1 && s[1] == '\n')
-				continue;
-			*t++ = c;
-		}
-		buflen = t - buf;
-
-/* Do we need to use quoted-printable? */
-/* Perhaps this hshould read (nacount > 0) */
+// Do we need to use quoted-printable?
+// Perhaps this hshould read (nacount > 0)
 		if (nacount * 20 > buflen || nullcount || longline) {
 			char *newbuf;
 			int l, colno = 0, space = 0;
@@ -503,7 +515,7 @@ empty:
 				    c == '=' ||
 				    c == '\xff' ||
 				    ((c == ' ' || c == '\t') &&
-				     s < v - 1 && s[1] == '\n')) {
+				     s < v - 1 && (s[1] == '\n' || s[1] == '\r'))) {
 					char expand[4];
 					sprintf(expand, "=%02X", (uchar) c);
 					stringAndString(&newbuf, &l, expand);
@@ -512,7 +524,7 @@ empty:
 					stringAndChar(&newbuf, &l, c);
 					++colno;
 				}
-				if (c == '\n') {
+				if (c == '\n' || c == '\r') {
 					colno = space = 0;
 					continue;
 				}
@@ -523,7 +535,7 @@ empty:
 				if (s == v - 1)
 					continue;
 /* If newline's coming up anyways, don't force another one. */
-				if (s[1] == '\n')
+				if (s[1] == '\n' || s[1] == '\r')
 					continue;
 				i = l;
 				if (!space || space == i) {
@@ -562,7 +574,7 @@ success:
 freefail:
 	nzFree(buf);
 	return false;
-}				/* encodeAttachment */
+}
 
 static char *mailTimeString(void)
 {
@@ -592,7 +604,7 @@ This is an internet standard, it's suppose to be English.
 	strftime(buf + 12, sizeof(buf) - 12, "%Y %H:%M:%S %z", cur_tm);
 
 	return buf;
-}				/* mailTimeString */
+}
 
 static char *messageTimeID(void)
 {
@@ -605,7 +617,7 @@ static char *messageTimeID(void)
 		cur_tm->tm_year + 1900, cur_tm->tm_mon, cur_tm->tm_mday,
 		cur_tm->tm_hour, cur_tm->tm_min, cur_tm->tm_sec);
 	return buf;
-}				/* messageTimeID */
+}
 
 static void appendAttachment(const char *s, char **out, int *l)
 {
@@ -629,14 +641,14 @@ static void appendAttachment(const char *s, char **out, int *l)
 	}
 /* Small bug here - an attachment that is not base64 encoded,
  * and had no newline at the end, now has one. */
-}				/* appendAttachment */
+}
 
 char *makeBoundary(void)
 {
 	static char boundary[24];
 	sprintf(boundary, "nextpart-eb-%06d", rand() % 1000000);
 	return boundary;
-}				/* makeBoundary */
+}
 
 struct smtp_upload {
 	const char *data;
@@ -665,7 +677,7 @@ static size_t smtp_upload_callback(char *buffer_for_curl, size_t size,
 	cur_pos = upload->pos + to_send;
 	upload->pos = cur_pos;
 	return to_send;
-}				/* smtp_upload_callback */
+}
 
 static char *buildSMTPURL(const struct MACCOUNT *account)
 {
@@ -689,21 +701,33 @@ static char *buildSMTPURL(const struct MACCOUNT *account)
 		i_printfExit(MSG_NoMem);
 
 	return url;
-}				/* buildSMTPURL */
+}
 
 static struct curl_slist *buildRecipientSList(const char **recipients)
 {
 	struct curl_slist *recipient_slist = NULL;
 	const char **r;
+	const char *r1, *r2, *r3;
 
 	for (r = recipients; *r; r++) {
-		recipient_slist = curl_slist_append(recipient_slist, *r);
+// do not include names here
+		r1 = *r;
+		r2 = strchr(r1, '<');
+		r3 = strchr(r1, '>');
+		if(r2 && r3) {
+// Yeah, this is const, and I'm casting it so I can change it,
+// but only for a second then I put it back.
+			*(char*)r3 = 0;
+			recipient_slist = curl_slist_append(recipient_slist, r2 + 1);
+			*(char*)r3 = '>';
+		} else
+			recipient_slist = curl_slist_append(recipient_slist, r1);
 		if (recipient_slist == NULL)
 			i_printfExit(MSG_NoMem);
 	}
 
 	return recipient_slist;
-}				/* buildRecipientSList */
+}
 
 static CURL *newSendmailHandle(const struct MACCOUNT *account,
 			       const char *outurl, const char *reply,
@@ -764,7 +788,7 @@ new_handle_cleanup:
 	}
 
 	return handle;
-}				/* newSendmailHandle */
+}
 
 typedef struct tagsmtp_upload {
 	const char *data;
@@ -806,7 +830,7 @@ smtp_cleanup:
 	curl_slist_free_all(recipient_slist);
 	nzFree(smtp_url);
 	return smtp_success;
-}				/* sendMailSMTP */
+}
 
 /* Send mail to the smtp server. */
 bool
@@ -817,9 +841,9 @@ sendMail(int account, const char **recipients, const char *body,
 	char *from, *fromiso, *reply;
 	const struct MACCOUNT *a, *ao, *localMail;
 	const char *s, *boundary;
-	char reccc[MAXRECAT];
+	char reccc[MAXRECAT + MAXCC];
 	char *t;
-	int nat, cx, i, j;
+	int nat, nrec, cx, i, j;
 	char *out = 0;
 	bool sendmail_success = false;
 	bool mustmime = false;
@@ -839,12 +863,18 @@ sendMail(int account, const char **recipients, const char *body,
 	doSignature = dosig;
 
 	nat = 0;		/* number of attachments */
-	if (attachments) {
-		while (attachments[nat])
-			++nat;
-	}
+	while (attachments[nat])
+		++nat;
+
+// attachments from the mail descriptor
+	for (j=0; a->cclist[j]; ++j)
+		if(a->cctype[j])
+			attachments[nat++] = a->cclist[j];
+	attachments[nat] = 0;
+
 	if (nat)
 		mustmime = true;
+
 	if (nalt && nalt < nat) {
 		setError(MSG_AttAlternate);
 		return false;
@@ -853,6 +883,16 @@ sendMail(int account, const char **recipients, const char *body,
 	if (!loadAddressBook())
 		return false;
 
+	nrec = 0;		/* number of recipients */
+	while (recipients[nrec])
+		++nrec;
+
+// recipients from the mail descriptor
+	for (j=0; a->cclist[j]; ++j)
+		if(!a->cctype[j])
+			recipients[nrec++] = a->cclist[j];
+	recipients[nrec] = 0;
+
 /* set copy flags */
 	for (j = 0; (s = recipients[j]); ++j) {
 		char cc = 0;
@@ -872,12 +912,14 @@ sendMail(int account, const char **recipients, const char *body,
 			continue;
 		t = 0;
 		for (i = 0; i < nads; ++i) {
-			const char *a = addressList[i].name;
+			const char *a = addressList[i].brief;
 			if (*a == '-' || *a == '!')
 				++a;
 			if (!stringEqual(s, a))
 				continue;
 			t = addressList[i].email;
+			if(addressList[i].fullname[0])
+				t = addressList[i].fullname;
 			debugPrint(3, " %s becomes %s", s, t);
 			break;
 		}
@@ -901,7 +943,7 @@ sendMail(int account, const char **recipients, const char *body,
 /* verify attachments are readable */
 	for (j = 0; (s = attachments[j]); ++j) {
 		if (!ismc && (cx = stringIsNum(s)) >= 0) {
-			if (!cxCompare(cx) || !cxActive(cx))
+			if (!cxCompare(cx) || !cxActive(cx, true))
 				return false;
 			if (!sessionList[cx].lw->dol) {
 				setError(MSG_AttSessionEmpty, cx);
@@ -1032,8 +1074,17 @@ this format, some or all of this message may not be legible.\r\n\r\n--");
 /* Also, suppress filename if this is an alternate presentation. */
 			if (!nalt && !strchr(s, '"')
 			    && (ismc || stringIsNum(s) < 0)) {
-				sprintf(serverLine, "; name=\"%s\"", s);
-				stringAndString(&out, &j, serverLine);
+// for security reasons, and convenience, don't present the absolute path.
+				const char *s2 = strrchr(s, '/');
+				if(s2) {
+					if(!*++s2)
+// attaching a directory?  This just shouldn't happen.
+						s2 = 0;
+				} else s2 = s;
+				if(s2) {
+					sprintf(serverLine, "; name=\"%s\"", s2);
+					stringAndString(&out, &j, serverLine);
+				}
 			}
 			sprintf(serverLine,
 				"%sContent-Transfer-Encoding: %s%s%s", eol, ce,
@@ -1054,7 +1105,7 @@ this format, some or all of this message may not be legible.\r\n\r\n--");
 	sendmail_success = sendMailSMTP(ao, reply, recipients, out);
 	nzFree(out);
 	return sendmail_success;
-}				/* sendMail */
+}
 
 bool validAccount(int n)
 {
@@ -1067,19 +1118,19 @@ bool validAccount(int n)
 		return false;
 	}
 	return true;
-}				/* validAccount */
+}
 
 bool sendMailCurrent(int sm_account, bool dosig)
 {
-	const char *reclist[MAXRECAT + 1];
+	const char *reclist[MAXRECAT + MAXCC + 1];
 	char *recmem;
-	const char *atlist[MAXRECAT + 1];
+	const char *atlist[MAXRECAT + MAXCC + 1];
 	char *atmem;
 	char *s, *t;
 	char cxbuf[4];
 	int lr, la, ln;
 	char *refline = 0;
-	int nrec = 0, nat = 0, nalt = 0;
+	int nrec, nat, nalt;
 	int account = localAccount;
 	int j;
 	bool rc = false;
@@ -1109,6 +1160,8 @@ bool sendMailCurrent(int sm_account, bool dosig)
 	if (!validAccount(account))
 		return false;
 
+	nrec = nat = nalt = 0;
+
 	recmem = initString(&lr);
 	atmem = initString(&la);
 
@@ -1195,7 +1248,7 @@ bool sendMailCurrent(int sm_account, bool dosig)
 			continue;
 		}
 
-		if (memEqualCI(line, "subject:", 8)) {
+		if (memEqualCI(line, "subject:", 8) || memEqualCI(line, "sub:", 4)) {
 			while (*line == ' ' || *line == '\t')
 				++line;
 			subj = true;
@@ -1206,6 +1259,7 @@ bool sendMailCurrent(int sm_account, bool dosig)
 
 	if (sm_account)
 		account = sm_account;
+
 	if (!subj) {
 		setError(((ln > cw->dol) + MSG_MailFirstLine), ln);
 		goto done;
@@ -1222,6 +1276,7 @@ bool sendMailCurrent(int sm_account, bool dosig)
 		reclist[j] = s;
 	}
 	reclist[j] = 0;
+
 	for (s = atmem, j = 0; *s; s = t + 1, ++j) {
 		t = strchr(s, '\n');
 		*t = 0;
@@ -1238,7 +1293,7 @@ done:
 	nzFree(atmem);
 	if (!rc && intFlag)
 		setError(MSG_Interrupted);
-	if (rc)
+	if (rc && debugLevel >= 1)
 		i_puts(MSG_OK);
 	return rc;
-}				/* sendMailCurrent */
+}
