@@ -24,6 +24,7 @@ char *debugFileName;
 bool demin = false;
 bool gotimers = true;
 bool uvw;
+int timerspeed = 1;
 char *downDir, *home;
 
 /*********************************************************************
@@ -41,7 +42,7 @@ void *allocMem(size_t n)
 	if (!(s = malloc(n)))
 		i_printfExit(MSG_MemAllocError, n);
 	return s;
-}				/* allocMem */
+}
 
 void *allocZeroMem(size_t n)
 {
@@ -51,7 +52,7 @@ void *allocZeroMem(size_t n)
 	if (!(s = calloc(n, 1)))
 		i_printfExit(MSG_MemCallocError, n);
 	return s;
-}				/* allocZeroMem */
+}
 
 void *reallocMem(void *p, size_t n)
 {
@@ -70,35 +71,35 @@ void *reallocMem(void *p, size_t n)
 	if (!(s = realloc(p, n)))
 		i_printfExit(MSG_ErrorRealloc, n);
 	return s;
-}				/* reallocMem */
+}
 
 /* When you know the allocated thing is a string. */
 char *allocString(size_t n)
 {
 	return (char *)allocMem(n);
-}				/* allocString */
+}
 
 char *allocZeroString(size_t n)
 {
 	return (char *)allocZeroMem(n);
-}				/* allocZeroString */
+}
 
 char *reallocString(void *p, size_t n)
 {
 	return (char *)reallocMem(p, n);
-}				/* reallocString */
+}
 
 void nzFree(void *s)
 {
 	if (s && s != emptyString)
 		free(s);
-}				/* nzFree */
+}
 
 /* some compilers care whether it's void * or const void * */
 void cnzFree(const void *v)
 {
 	nzFree((void *)v);
-}				/* cnzFree */
+}
 
 char *appendString(char *s, const char *p)
 {
@@ -107,7 +108,7 @@ char *appendString(char *s, const char *p)
 	s = reallocString(s, slen + plen + 1);
 	strcpy(s + slen, p);
 	return s;
-}				/* appendstring */
+}
 
 char *prependString(char *s, const char *p)
 {
@@ -118,7 +119,7 @@ char *prependString(char *s, const char *p)
 	strcpy(t + plen, s);
 	nzFree(s);
 	return t;
-}				/* prependstring */
+}
 
 void skipWhite(const char **s)
 {
@@ -126,7 +127,7 @@ void skipWhite(const char **s)
 	while (isspaceByte(*t))
 		++t;
 	*s = t;
-}				/* skipWhite */
+}
 
 void trimWhite(char *s)
 {
@@ -137,7 +138,7 @@ void trimWhite(char *s)
 	while (l && isspaceByte(s[l - 1]))
 		--l;
 	s[l] = 0;
-}				/* trimWhite */
+}
 
 void stripWhite(char *s)
 {
@@ -146,7 +147,7 @@ void stripWhite(char *s)
 	if (t > s)
 		strmove(s, t);
 	trimWhite(s);
-}				/* stripWhite */
+}
 
 /* compress white space */
 void spaceCrunch(char *s, bool onespace, bool unprint)
@@ -360,7 +361,7 @@ int stringIsNum(const char *s)
 	if (*s)
 		return -1;
 	return n;
-}				/* stringIsNum */
+}
 
 bool stringIsDate(const char *s)
 {
@@ -390,7 +391,7 @@ bool stringIsDate(const char *s)
 	if (*s)
 		return false;
 	return true;
-}				/* stringIsDate */
+}
 
 bool stringIsFloat(const char *s, double *dp)
 {
@@ -399,7 +400,7 @@ bool stringIsFloat(const char *s, double *dp)
 	if (*t)
 		return false;	/* extra stuff at the end */
 	return true;
-}				/* stringIsFloat */
+}
 
 bool memEqualCI(const char *s, const char *t, int len)
 {
@@ -419,18 +420,7 @@ bool memEqualCI(const char *s, const char *t, int len)
 		++s, ++t;
 	}
 	return true;
-}				/* memEqualCI */
-
-char *strstrCI(const char *base, const char *search)
-{
-	int l = strlen(search);
-	while (*base) {
-		if (memEqualCI(base, search, l))
-			return (char *)base;
-		++base;
-	}
-	return 0;
-}				/* strstrCI */
+}
 
 bool stringEqual(const char *s, const char *t)
 {
@@ -442,7 +432,7 @@ bool stringEqual(const char *s, const char *t)
 	if (strcmp(s, t))
 		return false;
 	return true;
-}				/* stringEqual */
+}
 
 bool stringEqualCI(const char *s, const char *t)
 {
@@ -467,7 +457,19 @@ bool stringEqualCI(const char *s, const char *t)
 	if (*t)
 		return false;
 	return true;
-}				/* stringEqualCI */
+}
+
+// a text line in the buffer isn't a string; you can't use strstr.
+// It ends in \n
+const char *stringInBufLine(const char *s, const char *t)
+{
+	int n = strlen(t);
+	for (; *s != '\n'; ++s) {
+		if (!strncmp(s, t, n))
+			return s;
+	}
+	return 0;
+}
 
 int stringInList(const char *const *list, const char *s)
 {
@@ -641,16 +643,16 @@ void setDebugFile(const char *name)
 	if (!name || !*name)
 		return;
 	debugFileName = cloneString(name);
-	debugFile = fopen(name, (whichproc == 'e' ? "w" : "a"));
+	debugFile = fopen(name, "w");
 	if (debugFile) {
 #ifndef _MSC_VER		// port setlinebuf(debugFile);, if required...
 		setlinebuf(debugFile);
 #else
 		;
 #endif // !_MSC_VER
-	} else if (whichproc == 'e')
+	} else
 		printf("cannot create %s\n", name);
-}				/* setDebugFile */
+}
 
 void nl(void)
 {
@@ -1202,7 +1204,7 @@ p:
 	}
 
 	return buf;
-}				/* lsattr */
+}
 
 #ifdef DOSLIKE
 void ttySaveSettings(void)
@@ -1219,13 +1221,13 @@ void ttySaveSettings(void)
 		if (tcgetattr(0, &savettybuf))
 			i_printfExit(MSG_IoctlError);
 	}
-}				/* ttySaveSettings */
+}
 
-static void ttyRestoreSettings(void)
+void ttyRestoreSettings(void)
 {
 	if (isInteractive)
 		tcsetattr(0, TCSANOW, &savettybuf);
-}				/* ttyRestoreSettings */
+}
 
 /* put the tty in raw mode.
  * Review your Unix manual on termio.
@@ -1234,7 +1236,7 @@ static void ttyRestoreSettings(void)
  * min>0 time=0:  block until min chars are received.
  * min=0 time>0:  return 1 char, or 0 if the timer expires.
  * min=0 time=0:  nonblocking, return whatever chars have been received. */
-static void ttyRaw(int charcount, int timeout, bool isecho)
+void ttyRaw(int charcount, int timeout, bool isecho)
 {
 	struct termios buf = savettybuf;	/* structure copy */
 	buf.c_cc[VMIN] = charcount;
@@ -1243,7 +1245,7 @@ static void ttyRaw(int charcount, int timeout, bool isecho)
 	if (isecho)
 		buf.c_lflag |= ECHO;
 	tcsetattr(0, TCSANOW, &buf);
-}				/* ttyRaw */
+}
 #endif // #ifdef DOSLIKE y/n
 
 void ttySetEcho(bool enable_echo)
@@ -1276,7 +1278,7 @@ int getche(void)
 	read(0, &c, 1);
 	ttyRestoreSettings();
 	return c;
-}				/* getche */
+}
 
 int getch(void)
 {
@@ -1286,7 +1288,7 @@ int getch(void)
 	read(0, &c, 1);
 	ttyRestoreSettings();
 	return c;
-}				/* getche */
+}
 
 #endif // #ifndef DOSLIKE
 
@@ -1302,7 +1304,7 @@ char getLetter(const char *s)
 	}
 	printf("%c", c);
 	return c;
-}				/* getLetter */
+}
 
 /* Parameters: message, default file name, must this file be new,
  * and can we except an input of white space,
@@ -1351,7 +1353,7 @@ char *getFileName(int msg, const char *defname, bool isnew, bool ws)
 		}
 		return p;
 	}
-}				/* getFileName */
+}
 
 /* Protect a filename from expansion by the shell */
 static const char shellmeta[] = "\\\n\t |&;<>(){}#'\"~$*?";
@@ -1493,7 +1495,7 @@ static bool envExpand(const char *line, const char **expanded)
 	l = 0;
 	for (s = line + 1; isalnum(*s) || *s == '_'; ++s)
 		++l;
-	if (l >= sizeof(var1) || isdigit(line[1]) || (*s && *s != '/')) {
+	if ((unsigned)l >= sizeof(var1) || isdigit(line[1]) || (*s && *s != '/')) {
 /* invalid syntax, put things back */
 		s = line;
 		goto dollars;
@@ -1519,7 +1521,7 @@ static bool envExpand(const char *line, const char **expanded)
 		goto dollars;
 	}
 	l = strlen(udir);
-	if (l >= sizeof(varline))
+	if ((unsigned)l >= sizeof(varline))
 		goto longline;
 	strcpy(varline, udir);
 	t = varline + l;
@@ -1651,7 +1653,19 @@ bool envFile(const char *line, const char **expanded)
 	return true;
 #endif // #ifdef DOSLIKE y/n
 
-}				/* envFile */
+}
+
+// Like the above, but string is allocated, and errors are printed
+// This is only used to expand pathnames in .ebrc
+char *envFileAlloc(const char *line)
+{
+	const char *line2;
+	if(envFile(line, &line2))
+		return cloneString(line2);
+	showError();
+	setError(-1);
+	return 0;
+}
 
 /* Call the above routine if filename contains a  slash,
  * or prepend the download directory if it does not.
@@ -1672,7 +1686,7 @@ bool envFileDown(const char *line, const char **expanded)
 	sprintf(line2, "%s/%s", downDir, line);
 	*expanded = line2;
 	return true;
-}				/* envFileDown */
+}
 
 FILE *efopen(const char *name, const char *mode)
 {
@@ -1696,7 +1710,7 @@ FILE *efopen(const char *name, const char *mode)
 	else
 		i_printfExit(MSG_InvalidFopen, mode);
 	return 0;
-}				/* efopen */
+}
 
 int eopen(const char *name, int mode, int perms)
 {
@@ -1709,7 +1723,7 @@ int eopen(const char *name, int mode, int perms)
 	else
 		i_printfExit(MSG_OpenFail, name);
 	return -1;
-}				/* eopen */
+}
 
 void appendFile(const char *fname, const char *message, ...)
 {
@@ -1721,7 +1735,7 @@ void appendFile(const char *fname, const char *message, ...)
 	va_end(p);
 	fprintf(f, "\n");
 	fclose(f);
-}				/* appendFile */
+}
 
 /* like the above, but no formatting */
 void appendFileNF(const char *filename, const char *msg)
@@ -1729,18 +1743,18 @@ void appendFileNF(const char *filename, const char *msg)
 	FILE *f = efopen(filename, "a");
 	fprintf(f, "%s\n", msg);
 	fclose(f);
-}				/* appendFileNF */
+}
 
 /* Wrapper around system(). */
 int eb_system(const char *cmd, bool print_on_success)
 {
 	int system_ret = system(cmd);
 	if (system_ret != -1) {
-		if (print_on_success)
+		if (print_on_success && debugLevel > 0)
 			i_puts(MSG_OK);
 	} else {
 		i_printf(MSG_SystemCmdFail, system_ret);
 		nl();
 	}
 	return system_ret;
-}				/* eb_system */
+}
